@@ -17,3 +17,6 @@
 void get_config(void);
 void put_config(void);
 
+char *CtdlGetSysConfig(char *sysconfname);
+void CtdlPutSysConfig(char *sysconfname, char *sysconfdata);
+void validate_config(void);
