@@ -13,65 +13,14 @@
  * GNU General Public License for more details.
  */
 
-#include "sysdep.h"
-#include <stdlib.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <sys/socket.h>
-#include <syslog.h>
-#include <sys/syslog.h>
-/*
-#include <sys/syscall.h>
-*/
-
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#include <limits.h>
-#include <sys/resource.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <sys/un.h>
-#include <string.h>
-#include <pwd.h>
-#include <errno.h>
-#include <stdarg.h>
-#include <grp.h>
-#include <libcitadel.h>
-#include "citadel.h"
-#include "server.h"
-#include "sysdep_decls.h"
-#include "citserver.h"
-#include "support.h"
-#include "config.h"
-#include "database.h"
-#include "housekeeping.h"
-#include "modules/crypto/serv_crypto.h"	/* Needed for init_ssl, client_write_ssl, client_read_ssl, destruct_ssl */
+#include "ctdl_module.h"
+#include "serv_extensions.h"
 #include "ecrash.h"
 
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
-#include "ctdl_module.h"
-#include "threads.h"
+#include "citserver.h"
 #include "user_ops.h"
+#include "locate_host.h"
+#include "context.h"
 #include "control.h"
 
 int DebugSession = 0;
@@ -327,7 +276,12 @@ void terminate_all_sessions(void)
 	for (ccptr = ContextList; ccptr != NULL; ccptr = ccptr->next) {
 		if (ccptr->client_socket != -1)
 		{
-			CON_syslog(LOG_INFO, "terminate_all_sessions() is murdering %s", ccptr->curr_user);
+			if (ccptr->IO != NULL) {
+				CON_syslog(LOG_INFO, "terminate_all_sessions() is murdering %s IO[%ld]CC[%d]", ccptr->curr_user, ccptr->IO->ID, ccptr->cs_pid);
+			}
+			else {
+				CON_syslog(LOG_INFO, "terminate_all_sessions() is murdering %s CC[%d]", ccptr->curr_user, ccptr->cs_pid);
+			}
 			close(ccptr->client_socket);
 			ccptr->client_socket = -1;
 			killed++;
@@ -535,6 +489,99 @@ CitContext *CtdlGetContextArray(int *count)
 
 
 
+/*
+ * Back-end function for starting a session
+ */
+void begin_session(CitContext *con)
+{
+	/* 
+	 * Initialize some variables specific to our context.
+	 */
+	con->logged_in = 0;
+	con->internal_pgm = 0;
+	con->download_fp = NULL;
+	con->upload_fp = NULL;
+	con->cached_msglist = NULL;
+	con->cached_num_msgs = 0;
+	con->FirstExpressMessage = NULL;
+	time(&con->lastcmd);
+	time(&con->lastidle);
+	strcpy(con->lastcmdname, "    ");
+	strcpy(con->cs_clientname, "(unknown)");
+	strcpy(con->curr_user, NLI);
+	*con->net_node = '\0';
+	*con->fake_username = '\0';
+	*con->fake_hostname = '\0';
+	*con->fake_roomname = '\0';
+	*con->cs_clientinfo = '\0';
+	safestrncpy(con->cs_host, config.c_fqdn, sizeof con->cs_host);
+	safestrncpy(con->cs_addr, "", sizeof con->cs_addr);
+	con->cs_UDSclientUID = -1;
+	con->cs_host[sizeof con->cs_host - 1] = 0;
+	if (!CC->is_local_socket) {
+		locate_host(con->cs_host, sizeof con->cs_host,
+			con->cs_addr, sizeof con->cs_addr,
+			con->client_socket
+		);
+	}
+	else {
+		con->cs_host[0] = 0;
+		con->cs_addr[0] = 0;
+#ifdef HAVE_STRUCT_UCRED
+		{
+			/* as http://www.wsinnovations.com/softeng/articles/uds.html told us... */
+			struct ucred credentials;
+			socklen_t ucred_length = sizeof(struct ucred);
+			
+			/*fill in the user data structure */
+			if(getsockopt(con->client_socket, SOL_SOCKET, SO_PEERCRED, &credentials, &ucred_length)) {
+				syslog(LOG_NOTICE, "could obtain credentials from unix domain socket");
+				
+			}
+			else {		
+				/* the process ID of the process on the other side of the socket */
+				/* credentials.pid; */
+				
+				/* the effective UID of the process on the other side of the socket  */
+				con->cs_UDSclientUID = credentials.uid;
+				
+				/* the effective primary GID of the process on the other side of the socket */
+				/* credentials.gid; */
+				
+				/* To get supplemental groups, we will have to look them up in our account
+				   database, after a reverse lookup on the UID to get the account name.
+				   We can take this opportunity to check to see if this is a legit account.
+				*/
+				snprintf(con->cs_clientinfo, sizeof(con->cs_clientinfo),
+					 "PID: "F_PID_T"; UID: "F_UID_T"; GID: "F_XPID_T" ", 
+					 credentials.pid,
+					 credentials.uid,
+					 credentials.gid);
+			}
+		}
+#endif
+	}
+	con->cs_flags = 0;
+	con->upload_type = UPL_FILE;
+	con->dl_is_net = 0;
+
+	con->nologin = 0;
+	if (((config.c_maxsessions > 0)&&(num_sessions > config.c_maxsessions)) || CtdlWantSingleUser()) {
+		con->nologin = 1;
+	}
+
+	if (!CC->is_local_socket) {
+		syslog(LOG_NOTICE, "Session (%s) started from %s (%s).\n", con->ServiceName, con->cs_host, con->cs_addr);
+	}
+	else {
+		syslog(LOG_NOTICE, "Session (%s) started via local socket UID:%d.\n", con->ServiceName, con->cs_UDSclientUID);
+	}
+
+	/* Run any session startup routines registered by loadable modules */
+	PerformSessionHooks(EVT_START);
+}
+
+
 /*
  * This function fills in a context and its user field correctly
  * Then creates/loads that user
