@@ -149,6 +149,8 @@ struct CitContext {
 	long *cached_msglist;			/* results of the previous CtdlForEachMessage() */
 	int cached_num_msgs;
 
+	char vcard_updated_by_ldap;		/* !0 iff ldap changed the vcard, treat as aide update */
+
 	AsyncIO *IO;				/* if this session has AsyncIO going on... */
 };
 
@@ -210,13 +212,13 @@ static INLINE void become_session(CitContext *which_con) {
 extern int DebugSession;
 #define CONDBGLOG(LEVEL) if ((LEVEL != LOG_DEBUG) || (DebugSession != 0))
 
-#define CON_syslog(LEVEL, FORMAT, ...)				\
-	CONDBGLOG(LEVEL) syslog(LEVEL,				\
-				"Context: " FORMAT, __VA_ARGS__)
+#define CON_syslog(LEVEL, FORMAT, ...)					\
+	CONDBGLOG(LEVEL) syslog(LEVEL,					\
+				"%s Context: " FORMAT, IOSTR, __VA_ARGS__)
 
-#define CONM_syslog(LEVEL, FORMAT)			\
-	CONDBGLOG(LEVEL) syslog(LEVEL,			\
-				"Context: " FORMAT);
+#define CONM_syslog(LEVEL, FORMAT)				\
+	CONDBGLOG(LEVEL) syslog(LEVEL,				\
+				"%s Context: " FORMAT, IOSTR);
 
 
 #endif /* CONTEXT_H */
