@@ -27,16 +27,8 @@
 /*****************************************************************************/
 
 #include "sysdep.h"
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
-#include <ctype.h>
-#include <string.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <dirent.h>
-#include <syslog.h>
 #include <zlib.h>
 
 #ifdef HAVE_DB_H
@@ -54,17 +46,10 @@
 
 
 #include <libcitadel.h>
-#include "citadel.h"
-#include "server.h"
-#include "citserver.h"
-#include "database.h"
-#include "msgbase.h"
-#include "sysdep_decls.h"
-#include "threads.h"
-#include "config.h"
-#include "control.h"
 
 #include "ctdl_module.h"
+#include "control.h"
+#include "citserver.h"
 
 
 static DB *dbp[MAXCDB];		/* One DB handle for each Citadel database */
@@ -97,8 +82,10 @@ void cdb_verbose_log(const DB_ENV *dbenv, const char *msg)
 /* Verbose logging callback */
 void cdb_verbose_err(const DB_ENV *dbenv, const char *errpfx, const char *msg)
 {
+	int *FOO = NULL;
 	syslog(LOG_ALERT, "DB: %s", msg);
 	cit_backtrace();
+	*FOO = 1;
 }
 
 
@@ -183,7 +170,7 @@ void cdb_check_handles(void)
 /*
  * Cull the database logs
  */
-static void cdb_cull_logs(void)
+void cdb_cull_logs(void)
 {
 	u_int32_t flags;
 	int ret;
@@ -219,15 +206,6 @@ static void cdb_cull_logs(void)
 	}
 }
 
-/*
- * Manually initiate log file cull.
- */
-void cmd_cull(char *argbuf) {
-	if (CtdlAccessCheck(ac_internal)) return;
-	cdb_cull_logs();
-	cprintf("%d Database log file cull completed.\n", CIT_OK);
-}
-
 
 /*
  * Request a checkpoint of the database.  Called once per minute by the thread manager.
@@ -435,7 +413,6 @@ void cdb_chmod_data(void) {
 	}
 
 	syslog(LOG_DEBUG, "open_databases() finished\n");
-	CtdlRegisterProtoHook(cmd_cull, "CULL", "Cull database logs");
 }
 
 
@@ -912,8 +889,14 @@ int SeentDebugEnabled = 0;
 #define DBGLOG(LEVEL) if ((LEVEL != LOG_DEBUG) || (SeentDebugEnabled != 0))
 #define SEENM_syslog(LEVEL, FORMAT)					\
 	DBGLOG(LEVEL) syslog(LEVEL,					\
-			     "IO[%ld]CC[%ld] SEEN[%s][%d] " FORMAT,	\
-			     ioid, ccid, Facility, cType)
+			     "%s[%ld]CC[%ld] SEEN[%s][%d] " FORMAT,	\
+			     IOSTR, ioid, ccid, Facility, cType)
+
+#define SEEN_syslog(LEVEL, FORMAT, ...)					\
+	DBGLOG(LEVEL) syslog(LEVEL,					\
+			     "%s[%ld]CC[%ld] SEEN[%s][%d] " FORMAT,	\
+			     IOSTR, ioid, ccid, Facility, cType,	\
+			     __VA_ARGS__)
 
 time_t CheckIfAlreadySeen(const char *Facility,
 			  StrBuf *guid,
@@ -929,24 +912,23 @@ time_t CheckIfAlreadySeen(const char *Facility,
 
 	if (cType != eWrite)
 	{
-		SEENM_syslog(LOG_DEBUG, "Loading");
+		SEEN_syslog(LOG_DEBUG, "Loading [%s]", ChrPtr(guid));
 		cdbut = cdb_fetch(CDB_USETABLE, SKEY(guid));
 		if ((cdbut != NULL) && (cdbut->ptr != NULL)) {
 			memcpy(&ut, cdbut->ptr,
 			       ((cdbut->len > sizeof(struct UseTable)) ?
 				sizeof(struct UseTable) : cdbut->len));
-			InDBTimeStamp = ut.ut_timestamp;
+			InDBTimeStamp = now - ut.ut_timestamp;
 
 			if (InDBTimeStamp < antiexpire)
 			{
-				SEENM_syslog(LOG_DEBUG, "Found - Not expired.");
+				SEEN_syslog(LOG_DEBUG, "Found - Not expired %ld < %ld", InDBTimeStamp, antiexpire);
 				cdb_free(cdbut);
 				return InDBTimeStamp;
 			}
 			else
 			{
-				SEENM_syslog(LOG_DEBUG, "Found - Expired.");
-				InDBTimeStamp = ut.ut_timestamp;
+				SEEN_syslog(LOG_DEBUG, "Found - Expired. %ld >= %ld", InDBTimeStamp, antiexpire);
 				cdb_free(cdbut);
 			}
 		}
@@ -964,7 +946,7 @@ time_t CheckIfAlreadySeen(const char *Facility,
 	memcpy(ut.ut_msgid, SKEY(guid));
 	ut.ut_timestamp = now;
 
-	SEENM_syslog(LOG_DEBUG, "Saving");
+	SEENM_syslog(LOG_DEBUG, "Saving new Timestamp");
 	/* rewrite the record anyway, to update the timestamp */
 	cdb_store(CDB_USETABLE,
 		  SKEY(guid),
@@ -975,6 +957,52 @@ time_t CheckIfAlreadySeen(const char *Facility,
 }
 
 
+void cmd_rsen(char *argbuf) {
+	char Token[SIZ];
+	long TLen;
+	char Time[SIZ];
+
+	struct UseTable ut;
+	struct cdbdata *cdbut;
+	
+	if (CtdlAccessCheck(ac_aide)) return;
+
+	TLen = extract_token(Token, argbuf, 1, '|', sizeof Token);
+	if (strncmp(argbuf, "GET", 3) == 0) {
+		cdbut = cdb_fetch(CDB_USETABLE, Token, TLen);
+		if (cdbut != NULL) {
+			memcpy(&ut, cdbut->ptr,
+			       ((cdbut->len > sizeof(struct UseTable)) ?
+				sizeof(struct UseTable) : cdbut->len));
+			
+			cprintf("%d %ld\n", CIT_OK, ut.ut_timestamp);
+		}
+		else {
+			cprintf("%d not found\n", ERROR + NOT_HERE);
+		}
+
+	}
+	else if (strncmp(argbuf, "SET", 3) == 0) {
+		memcpy(ut.ut_msgid, Token, TLen);
+		extract_token(Time, argbuf, 2, '|', sizeof Time);
+		ut.ut_timestamp = atol(Time);
+		cdb_store(CDB_USETABLE,
+			  Token, TLen,
+			  &ut, sizeof(struct UseTable) );
+		cprintf("%d token updated\n", CIT_OK);
+	}
+	else if (strncmp(argbuf, "DEL", 3) == 0) {
+		if (cdb_delete(CDB_USETABLE, Token, TLen))
+			cprintf("%d not found\n", ERROR + NOT_HERE);
+		else
+			cprintf("%d deleted.\n", CIT_OK);
+
+	}
+	else {
+		cprintf("%d Usage: [GET|SET|DEL]|Token|timestamp\n", ERROR);
+	}
+
+}
 void LogDebugEnableSeenEnable(const int n)
 {
 	SeentDebugEnabled = n;
@@ -985,6 +1013,7 @@ CTDL_MODULE_INIT(database)
 	if (!threading)
 	{
 		CtdlRegisterDebugFlagHook(HKEY("SeenDebug"), LogDebugEnableSeenEnable, &SeentDebugEnabled);
+		CtdlRegisterProtoHook(cmd_rsen, "RSEN", "manipulate Aggregators seen database");
 	}
 
 	/* return our module id for the log */
