@@ -13,18 +13,7 @@
 #include <pwd.h>
 #include <errno.h>
 #include <sys/types.h>
-
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
+#include <time.h>
 #include <sys/wait.h>
 #include <string.h>
 #include <limits.h>
@@ -194,7 +183,7 @@ void utf8ify_rfc822_string(char *buf) {
 		}
 		else if (!strcasecmp(encoding, "Q")) {	/**< quoted-printable */
 			size_t len;
-			long pos;
+			unsigned long pos;
 			
 			len = strlen(istr);
 			pos = 0;
@@ -326,8 +315,620 @@ int CtdlHostAlias(char *fqdn) {
 
 
 
+/*
+ * Determine whether a given Internet address belongs to the current user
+ */
+int CtdlIsMe(char *addr, int addr_buf_len)
+{
+	recptypes *recp;
+	int i;
+
+	recp = validate_recipients(addr, NULL, 0);
+	if (recp == NULL) return(0);
+
+	if (recp->num_local == 0) {
+		free_recipients(recp);
+		return(0);
+	}
+
+	for (i=0; i<recp->num_local; ++i) {
+		extract_token(addr, recp->recp_local, i, '|', addr_buf_len);
+		if (!strcasecmp(addr, CC->user.fullname)) {
+			free_recipients(recp);
+			return(1);
+		}
+	}
+
+	free_recipients(recp);
+	return(0);
+}
 
 
+/* If the last item in a list of recipients was truncated to a partial address,
+ * remove it completely in order to avoid choking libSieve
+ */
+void sanitize_truncated_recipient(char *str)
+{
+	if (!str) return;
+	if (num_tokens(str, ',') < 2) return;
+
+	int len = strlen(str);
+	if (len < 900) return;
+	if (len > 998) str[998] = 0;
+
+	char *cptr = strrchr(str, ',');
+	if (!cptr) return;
+
+	char *lptr = strchr(cptr, '<');
+	char *rptr = strchr(cptr, '>');
+
+	if ( (lptr) && (rptr) && (rptr > lptr) ) return;
+
+	*cptr = 0;
+}
+
+
+
+
+
+/*
+ * This function is self explanatory.
+ * (What can I say, I'm in a weird mood today...)
+ */
+void remove_any_whitespace_to_the_left_or_right_of_at_symbol(char *name)
+{
+	unsigned int i;
+
+	for (i = 0; i < strlen(name); ++i) {
+		if (name[i] == '@') {
+			while (isspace(name[i - 1]) && i > 0) {
+				strcpy(&name[i - 1], &name[i]);
+				--i;
+			}
+			while (isspace(name[i + 1])) {
+				strcpy(&name[i + 1], &name[i + 2]);
+			}
+		}
+	}
+}
+
+
+/*
+ * Aliasing for network mail.
+ * (Error messages have been commented out, because this is a server.)
+ */
+int alias(char *name)
+{				/* process alias and routing info for mail */
+	struct CitContext *CCC = CC;
+	FILE *fp;
+	int a, i;
+	char aaa[SIZ], bbb[SIZ];
+	char *ignetcfg = NULL;
+	char *ignetmap = NULL;
+	int at = 0;
+	char node[64];
+	char testnode[64];
+	char buf[SIZ];
+
+	char original_name[256];
+	safestrncpy(original_name, name, sizeof original_name);
+
+	striplt(name);
+	remove_any_whitespace_to_the_left_or_right_of_at_symbol(name);
+	stripallbut(name, '<', '>');
+
+	fp = fopen(file_mail_aliases, "r");
+	if (fp == NULL) {
+		fp = fopen("/dev/null", "r");
+	}
+	if (fp == NULL) {
+		return (MES_ERROR);
+	}
+	strcpy(aaa, "");
+	strcpy(bbb, "");
+	while (fgets(aaa, sizeof aaa, fp) != NULL) {
+		while (isspace(name[0]))
+			strcpy(name, &name[1]);
+		aaa[strlen(aaa) - 1] = 0;
+		strcpy(bbb, "");
+		for (a = 0; aaa[a] != '\0'; ++a) {
+			if (aaa[a] == ',') {
+				strcpy(bbb, &aaa[a + 1]);
+				aaa[a] = 0;
+				break;
+			}
+		}
+		if (!strcasecmp(name, aaa))
+			strcpy(name, bbb);
+	}
+	fclose(fp);
+
+	/* Hit the Global Address Book */
+	if (CtdlDirectoryLookup(aaa, name, sizeof aaa) == 0) {
+		strcpy(name, aaa);
+	}
+
+	if (strcasecmp(original_name, name)) {
+		MSG_syslog(LOG_INFO, "%s is being forwarded to %s\n", original_name, name);
+	}
+
+	/* Change "user @ xxx" to "user" if xxx is an alias for this host */
+	for (a=0; name[a] != '\0'; ++a) {
+		if (name[a] == '@') {
+			if (CtdlHostAlias(&name[a+1]) == hostalias_localhost) {
+				name[a] = 0;
+				MSG_syslog(LOG_INFO, "Changed to <%s>\n", name);
+				break;
+			}
+		}
+	}
+
+	/* determine local or remote type, see citadel.h */
+	at = haschar(name, '@');
+	if (at == 0) return(MES_LOCAL);		/* no @'s - local address */
+	if (at > 1) return(MES_ERROR);		/* >1 @'s - invalid address */
+	remove_any_whitespace_to_the_left_or_right_of_at_symbol(name);
+
+	/* figure out the delivery mode */
+	extract_token(node, name, 1, '@', sizeof node);
+
+	/* If there are one or more dots in the nodename, we assume that it
+	 * is an FQDN and will attempt SMTP delivery to the Internet.
+	 */
+	if (haschar(node, '.') > 0) {
+		return(MES_INTERNET);
+	}
+
+	/* Otherwise we look in the IGnet maps for a valid Citadel node.
+	 * Try directly-connected nodes first...
+	 */
+	ignetcfg = CtdlGetSysConfig(IGNETCFG);
+	for (i=0; i<num_tokens(ignetcfg, '\n'); ++i) {
+		extract_token(buf, ignetcfg, i, '\n', sizeof buf);
+		extract_token(testnode, buf, 0, '|', sizeof testnode);
+		if (!strcasecmp(node, testnode)) {
+			free(ignetcfg);
+			return(MES_IGNET);
+		}
+	}
+	free(ignetcfg);
+
+	/*
+	 * Then try nodes that are two or more hops away.
+	 */
+	ignetmap = CtdlGetSysConfig(IGNETMAP);
+	for (i=0; i<num_tokens(ignetmap, '\n'); ++i) {
+		extract_token(buf, ignetmap, i, '\n', sizeof buf);
+		extract_token(testnode, buf, 0, '|', sizeof testnode);
+		if (!strcasecmp(node, testnode)) {
+			free(ignetmap);
+			return(MES_IGNET);
+		}
+	}
+	free(ignetmap);
+
+	/* If we get to this point it's an invalid node name */
+	return (MES_ERROR);
+}
+
+
+
+/*
+ * Validate recipients, count delivery types and errors, and handle aliasing
+ * FIXME check for dupes!!!!!
+ *
+ * Returns 0 if all addresses are ok, ret->num_error = -1 if no addresses 
+ * were specified, or the number of addresses found invalid.
+ *
+ * Caller needs to free the result using free_recipients()
+ */
+recptypes *validate_recipients(const char *supplied_recipients, 
+			       const char *RemoteIdentifier, 
+			       int Flags) {
+	struct CitContext *CCC = CC;
+	recptypes *ret;
+	char *recipients = NULL;
+	char *org_recp;
+	char this_recp[256];
+	char this_recp_cooked[256];
+	char append[SIZ];
+	long len;
+	int num_recps = 0;
+	int i, j;
+	int mailtype;
+	int invalid;
+	struct ctdluser tempUS;
+	struct ctdlroom tempQR;
+	struct ctdlroom tempQR2;
+	int err = 0;
+	char errmsg[SIZ];
+	int in_quotes = 0;
+
+	/* Initialize */
+	ret = (recptypes *) malloc(sizeof(recptypes));
+	if (ret == NULL) return(NULL);
+
+	/* Set all strings to null and numeric values to zero */
+	memset(ret, 0, sizeof(recptypes));
+
+	if (supplied_recipients == NULL) {
+		recipients = strdup("");
+	}
+	else {
+		recipients = strdup(supplied_recipients);
+	}
+
+	/* Allocate some memory.  Yes, this allocates 500% more memory than we will
+	 * actually need, but it's healthier for the heap than doing lots of tiny
+	 * realloc() calls instead.
+	 */
+	len = strlen(recipients) + 1024;
+	ret->errormsg = malloc(len);
+	ret->recp_local = malloc(len);
+	ret->recp_internet = malloc(len);
+	ret->recp_ignet = malloc(len);
+	ret->recp_room = malloc(len);
+	ret->display_recp = malloc(len);
+	ret->recp_orgroom = malloc(len);
+	org_recp = malloc(len);
+
+	ret->errormsg[0] = 0;
+	ret->recp_local[0] = 0;
+	ret->recp_internet[0] = 0;
+	ret->recp_ignet[0] = 0;
+	ret->recp_room[0] = 0;
+	ret->recp_orgroom[0] = 0;
+	ret->display_recp[0] = 0;
+
+	ret->recptypes_magic = RECPTYPES_MAGIC;
+
+	/* Change all valid separator characters to commas */
+	for (i=0; !IsEmptyStr(&recipients[i]); ++i) {
+		if ((recipients[i] == ';') || (recipients[i] == '|')) {
+			recipients[i] = ',';
+		}
+	}
+
+	/* Now start extracting recipients... */
+
+	while (!IsEmptyStr(recipients)) {
+		for (i=0; i<=strlen(recipients); ++i) {
+			if (recipients[i] == '\"') in_quotes = 1 - in_quotes;
+			if ( ( (recipients[i] == ',') && (!in_quotes) ) || (recipients[i] == 0) ) {
+				safestrncpy(this_recp, recipients, i+1);
+				this_recp[i] = 0;
+				if (recipients[i] == ',') {
+					strcpy(recipients, &recipients[i+1]);
+				}
+				else {
+					strcpy(recipients, "");
+				}
+				break;
+			}
+		}
+
+		striplt(this_recp);
+		if (IsEmptyStr(this_recp))
+			break;
+		MSG_syslog(LOG_DEBUG, "Evaluating recipient #%d: %s\n", num_recps, this_recp);
+		++num_recps;
+
+		strcpy(org_recp, this_recp);
+		alias(this_recp);
+		alias(this_recp);
+		mailtype = alias(this_recp);
+
+		for (j = 0; !IsEmptyStr(&this_recp[j]); ++j) {
+			if (this_recp[j]=='_') {
+				this_recp_cooked[j] = ' ';
+			}
+			else {
+				this_recp_cooked[j] = this_recp[j];
+			}
+		}
+		this_recp_cooked[j] = '\0';
+		invalid = 0;
+		errmsg[0] = 0;
+		switch(mailtype) {
+		case MES_LOCAL:
+			if (!strcasecmp(this_recp, "sysop")) {
+				++ret->num_room;
+				strcpy(this_recp, config.c_aideroom);
+				if (!IsEmptyStr(ret->recp_room)) {
+					strcat(ret->recp_room, "|");
+				}
+				strcat(ret->recp_room, this_recp);
+			}
+			else if ( (!strncasecmp(this_recp, "room_", 5))
+				  && (!CtdlGetRoom(&tempQR, &this_recp_cooked[5])) ) {
+
+				/* Save room so we can restore it later */
+				tempQR2 = CCC->room;
+				CCC->room = tempQR;
+					
+				/* Check permissions to send mail to this room */
+				err = CtdlDoIHavePermissionToPostInThisRoom(
+					errmsg, 
+					sizeof errmsg, 
+					RemoteIdentifier,
+					Flags,
+					0			/* 0 = not a reply */
+					);
+				if (err)
+				{
+					++ret->num_error;
+					invalid = 1;
+				} 
+				else {
+					++ret->num_room;
+					if (!IsEmptyStr(ret->recp_room)) {
+						strcat(ret->recp_room, "|");
+					}
+					strcat(ret->recp_room, &this_recp_cooked[5]);
+
+					if (!IsEmptyStr(ret->recp_orgroom)) {
+						strcat(ret->recp_orgroom, "|");
+					}
+					strcat(ret->recp_orgroom, org_recp);
+
+				}
+					
+				/* Restore room in case something needs it */
+				CCC->room = tempQR2;
+
+			}
+			else if (CtdlGetUser(&tempUS, this_recp) == 0) {
+				++ret->num_local;
+				strcpy(this_recp, tempUS.fullname);
+				if (!IsEmptyStr(ret->recp_local)) {
+					strcat(ret->recp_local, "|");
+				}
+				strcat(ret->recp_local, this_recp);
+			}
+			else if (CtdlGetUser(&tempUS, this_recp_cooked) == 0) {
+				++ret->num_local;
+				strcpy(this_recp, tempUS.fullname);
+				if (!IsEmptyStr(ret->recp_local)) {
+					strcat(ret->recp_local, "|");
+				}
+				strcat(ret->recp_local, this_recp);
+			}
+			else {
+				++ret->num_error;
+				invalid = 1;
+			}
+			break;
+		case MES_INTERNET:
+			/* Yes, you're reading this correctly: if the target
+			 * domain points back to the local system or an attached
+			 * Citadel directory, the address is invalid.  That's
+			 * because if the address were valid, we would have
+			 * already translated it to a local address by now.
+			 */
+			if (IsDirectory(this_recp, 0)) {
+				++ret->num_error;
+				invalid = 1;
+			}
+			else {
+				++ret->num_internet;
+				if (!IsEmptyStr(ret->recp_internet)) {
+					strcat(ret->recp_internet, "|");
+				}
+				strcat(ret->recp_internet, this_recp);
+			}
+			break;
+		case MES_IGNET:
+			++ret->num_ignet;
+			if (!IsEmptyStr(ret->recp_ignet)) {
+				strcat(ret->recp_ignet, "|");
+			}
+			strcat(ret->recp_ignet, this_recp);
+			break;
+		case MES_ERROR:
+			++ret->num_error;
+			invalid = 1;
+			break;
+		}
+		if (invalid) {
+			if (IsEmptyStr(errmsg)) {
+				snprintf(append, sizeof append, "Invalid recipient: %s", this_recp);
+			}
+			else {
+				snprintf(append, sizeof append, "%s", errmsg);
+			}
+			if ( (strlen(ret->errormsg) + strlen(append) + 3) < SIZ) {
+				if (!IsEmptyStr(ret->errormsg)) {
+					strcat(ret->errormsg, "; ");
+				}
+				strcat(ret->errormsg, append);
+			}
+		}
+		else {
+			if (IsEmptyStr(ret->display_recp)) {
+				strcpy(append, this_recp);
+			}
+			else {
+				snprintf(append, sizeof append, ", %s", this_recp);
+			}
+			if ( (strlen(ret->display_recp)+strlen(append)) < SIZ) {
+				strcat(ret->display_recp, append);
+			}
+		}
+	}
+	free(org_recp);
+
+	if ((ret->num_local + ret->num_internet + ret->num_ignet +
+	     ret->num_room + ret->num_error) == 0) {
+		ret->num_error = (-1);
+		strcpy(ret->errormsg, "No recipients specified.");
+	}
+
+	MSGM_syslog(LOG_DEBUG, "validate_recipients()\n");
+	MSG_syslog(LOG_DEBUG, " local: %d <%s>\n", ret->num_local, ret->recp_local);
+	MSG_syslog(LOG_DEBUG, "  room: %d <%s>\n", ret->num_room, ret->recp_room);
+	MSG_syslog(LOG_DEBUG, "  inet: %d <%s>\n", ret->num_internet, ret->recp_internet);
+	MSG_syslog(LOG_DEBUG, " ignet: %d <%s>\n", ret->num_ignet, ret->recp_ignet);
+	MSG_syslog(LOG_DEBUG, " error: %d <%s>\n", ret->num_error, ret->errormsg);
+
+	free(recipients);
+	return(ret);
+}
+
+
+/*
+ * Destructor for recptypes
+ */
+void free_recipients(recptypes *valid) {
+
+	if (valid == NULL) {
+		return;
+	}
+
+	if (valid->recptypes_magic != RECPTYPES_MAGIC) {
+		struct CitContext *CCC = CC;
+		MSGM_syslog(LOG_EMERG, "Attempt to call free_recipients() on some other data type!\n");
+		abort();
+	}
+
+	if (valid->errormsg != NULL)		free(valid->errormsg);
+	if (valid->recp_local != NULL)		free(valid->recp_local);
+	if (valid->recp_internet != NULL)	free(valid->recp_internet);
+	if (valid->recp_ignet != NULL)		free(valid->recp_ignet);
+	if (valid->recp_room != NULL)		free(valid->recp_room);
+	if (valid->recp_orgroom != NULL)        free(valid->recp_orgroom);
+	if (valid->display_recp != NULL)	free(valid->display_recp);
+	if (valid->bounce_to != NULL)		free(valid->bounce_to);
+	if (valid->envelope_from != NULL)	free(valid->envelope_from);
+	if (valid->sending_room != NULL)	free(valid->sending_room);
+	free(valid);
+}
+
+
+char *qp_encode_email_addrs(char *source)
+{
+	struct CitContext *CCC = CC;
+	char *user, *node, *name;
+	const char headerStr[] = "=?UTF-8?Q?";
+	char *Encoded;
+	char *EncodedName;
+	char *nPtr;
+	int need_to_encode = 0;
+	long SourceLen;
+	long EncodedMaxLen;
+	long nColons = 0;
+	long *AddrPtr;
+	long *AddrUtf8;
+	long nAddrPtrMax = 50;
+	long nmax;
+	int InQuotes = 0;
+	int i, n;
+
+	if (source == NULL) return source;
+	if (IsEmptyStr(source)) return source;
+	if (MessageDebugEnabled != 0) cit_backtrace();
+	MSG_syslog(LOG_DEBUG, "qp_encode_email_addrs: [%s]\n", source);
+
+	AddrPtr = malloc (sizeof (long) * nAddrPtrMax);
+	AddrUtf8 = malloc (sizeof (long) * nAddrPtrMax);
+	memset(AddrUtf8, 0, sizeof (long) * nAddrPtrMax);
+	*AddrPtr = 0;
+	i = 0;
+	while (!IsEmptyStr (&source[i])) {
+		if (nColons >= nAddrPtrMax){
+			long *ptr;
+
+			ptr = (long *) malloc(sizeof (long) * nAddrPtrMax * 2);
+			memcpy (ptr, AddrPtr, sizeof (long) * nAddrPtrMax);
+			free (AddrPtr), AddrPtr = ptr;
+
+			ptr = (long *) malloc(sizeof (long) * nAddrPtrMax * 2);
+			memset(&ptr[nAddrPtrMax], 0, 
+			       sizeof (long) * nAddrPtrMax);
+
+			memcpy (ptr, AddrUtf8, sizeof (long) * nAddrPtrMax);
+			free (AddrUtf8), AddrUtf8 = ptr;
+			nAddrPtrMax *= 2;				
+		}
+		if (((unsigned char) source[i] < 32) || 
+		    ((unsigned char) source[i] > 126)) {
+			need_to_encode = 1;
+			AddrUtf8[nColons] = 1;
+		}
+		if (source[i] == '"')
+			InQuotes = !InQuotes;
+		if (!InQuotes && source[i] == ',') {
+			AddrPtr[nColons] = i;
+			nColons++;
+		}
+		i++;
+	}
+	if (need_to_encode == 0) {
+		free(AddrPtr);
+		free(AddrUtf8);
+		return source;
+	}
+
+	SourceLen = i;
+	EncodedMaxLen = nColons * (sizeof(headerStr) + 3) + SourceLen * 3;
+	Encoded = (char*) malloc (EncodedMaxLen);
+
+	for (i = 0; i < nColons; i++)
+		source[AddrPtr[i]++] = '\0';
+	/* TODO: if libidn, this might get larger*/
+	user = malloc(SourceLen + 1);
+	node = malloc(SourceLen + 1);
+	name = malloc(SourceLen + 1);
+
+	nPtr = Encoded;
+	*nPtr = '\0';
+	for (i = 0; i < nColons && nPtr != NULL; i++) {
+		nmax = EncodedMaxLen - (nPtr - Encoded);
+		if (AddrUtf8[i]) {
+			process_rfc822_addr(&source[AddrPtr[i]], 
+					    user,
+					    node,
+					    name);
+			/* TODO: libIDN here ! */
+			if (IsEmptyStr(name)) {
+				n = snprintf(nPtr, nmax, 
+					     (i==0)?"%s@%s" : ",%s@%s",
+					     user, node);
+			}
+			else {
+				EncodedName = rfc2047encode(name, strlen(name));			
+				n = snprintf(nPtr, nmax, 
+					     (i==0)?"%s <%s@%s>" : ",%s <%s@%s>",
+					     EncodedName, user, node);
+				free(EncodedName);
+			}
+		}
+		else { 
+			n = snprintf(nPtr, nmax, 
+				     (i==0)?"%s" : ",%s",
+				     &source[AddrPtr[i]]);
+		}
+		if (n > 0 )
+			nPtr += n;
+		else { 
+			char *ptr, *nnPtr;
+			ptr = (char*) malloc(EncodedMaxLen * 2);
+			memcpy(ptr, Encoded, EncodedMaxLen);
+			nnPtr = ptr + (nPtr - Encoded), nPtr = nnPtr;
+			free(Encoded), Encoded = ptr;
+			EncodedMaxLen *= 2;
+			i--; /* do it once more with properly lengthened buffer */
+		}
+	}
+	for (i = 0; i < nColons; i++)
+		source[--AddrPtr[i]] = ',';
+
+	free(user);
+	free(node);
+	free(name);
+	free(AddrUtf8);
+	free(AddrPtr);
+	return Encoded;
+}
 
 
 /*
@@ -379,11 +980,16 @@ void unfold_rfc822_field(char **field, char **FieldEnd)
 	     sField < pFieldEnd; 
 	     pField++, sField++)
 	{
-		if ((*sField=='\r') || (*sField=='\n')) {
-		    sField++;
-		    if  (*sField == '\n')
-			sField++;
-		    *pField = *sField;
+		if ((*sField=='\r') || (*sField=='\n'))
+		{
+			int Offset = 1;
+			while (((*(sField + Offset) == '\r') ||
+				(*(sField + Offset) == '\n') ||
+				(isspace(*(sField + Offset)))) && 
+			       (sField + Offset < pFieldEnd))
+				Offset ++;
+			sField += Offset;
+			*pField = *sField;
 		}
 		else {
 			if (*sField=='\"') quote = 1 - quote;
@@ -433,11 +1039,15 @@ void process_rfc822_addr(const char *rfc822, char *user, char *node, char *name)
 		strcpy(name, &name[1]);
 
 	/* and anything to the right of a @ or % */
-	for (a = 0; a < strlen(name); ++a) {
-		if (name[a] == '@')
+	for (a = 0; name[a] != '\0'; ++a) {
+		if (name[a] == '@') {
 			name[a] = 0;
-		if (name[a] == '%')
+			break;
+		}
+		if (name[a] == '%') {
 			name[a] = 0;
+			break;
+		}
 	}
 
 	/* but if there are parentheses, that changes the rules... */
@@ -453,9 +1063,11 @@ void process_rfc822_addr(const char *rfc822, char *user, char *node, char *name)
 			strcpy(&name[0], &name[1]);
 		}
 		strcpy(&name[0], &name[1]);
-		for (a = 0; a < strlen(name); ++a)
-			if (name[a] == 34)
+		for (a = 0; name[a] != '\0'; ++a)
+			if (name[a] == 34) {
 				name[a] = 0;
+				break;
+			}
 	}
 	/* extract user id */
 	strcpy(user, rfc822);
@@ -473,11 +1085,15 @@ void process_rfc822_addr(const char *rfc822, char *user, char *node, char *name)
 		strcpy(user, &user[1]);
 
 	/* and anything to the right of a @ or % */
-	for (a = 0; a < strlen(user); ++a) {
-		if (user[a] == '@')
+	for (a = 0; user[a] != '\0'; ++a) {
+		if (user[a] == '@') {
 			user[a] = 0;
-		if (user[a] == '%')
+			break;
+		}
+		if (user[a] == '%') {
 			user[a] = 0;
+			break;
+		}
 	}
 
 
@@ -516,9 +1132,11 @@ void process_rfc822_addr(const char *rfc822, char *user, char *node, char *name)
 			strcpy(node, &node[1]);
 	
 		/* now get rid of the user portion of a node!user string */
-		for (a = 0; a < strlen(node); ++a)
-			if (node[a] == '!')
+		for (a = 0; node[a] != '\0'; ++a)
+			if (node[a] == '!') {
 				node[a] = 0;
+				break;
+			}
 	}
 
 	/* strip leading and trailing spaces in all strings */
@@ -553,7 +1171,6 @@ int convert_field(struct CtdlMessage *msg, const char *beg, const char *end) {
 	int i;
 	const char *colonpos = NULL;
 	int processed = 0;
-	char buf[SIZ];
 	char user[1024];
 	char node[1024];
 	char name[1024];
@@ -589,100 +1206,100 @@ int convert_field(struct CtdlMessage *msg, const char *beg, const char *end) {
 	if (!strcasecmp(key, "Date")) {
 		parsed_date = parsedate(value);
 		if (parsed_date < 0L) parsed_date = time(NULL);
-		snprintf(buf, sizeof buf, "%ld", (long)parsed_date );
-		if (msg->cm_fields['T'] == NULL)
-			msg->cm_fields['T'] = strdup(buf);
+
+		if (CM_IsEmpty(msg, eTimestamp))
+			CM_SetFieldLONG(msg, eTimestamp, parsed_date);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "From")) {
 		process_rfc822_addr(value, user, node, name);
 		syslog(LOG_DEBUG, "Converted to <%s@%s> (%s)\n", user, node, name);
-		snprintf(addr, sizeof addr, "%s@%s", user, node);
-		if (msg->cm_fields['A'] == NULL)
-			msg->cm_fields['A'] = strdup(name);
-		if (msg->cm_fields['F'] == NULL)
-			msg->cm_fields['F'] = strdup(addr);
+		snprintf(addr, sizeof(addr), "%s@%s", user, node);
+		if (CM_IsEmpty(msg, eAuthor))
+			CM_SetField(msg, eAuthor, name, strlen(name));
+		if (CM_IsEmpty(msg, erFc822Addr))
+			CM_SetField(msg, erFc822Addr, addr, strlen(addr));
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "Subject")) {
-		if (msg->cm_fields['U'] == NULL)
-			msg->cm_fields['U'] = strndup(value, valuelen);
+		if (CM_IsEmpty(msg, eMsgSubject))
+			CM_SetField(msg, eMsgSubject, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "List-ID")) {
-		if (msg->cm_fields['L'] == NULL)
-			msg->cm_fields['L'] = strndup(value, valuelen);
+		if (CM_IsEmpty(msg, eListID))
+			CM_SetField(msg, eListID, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "To")) {
-		if (msg->cm_fields['R'] == NULL)
-			msg->cm_fields['R'] = strndup(value, valuelen);
+		if (CM_IsEmpty(msg, eRecipient))
+			CM_SetField(msg, eRecipient, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "CC")) {
-		if (msg->cm_fields['Y'] == NULL)
-			msg->cm_fields['Y'] = strndup(value, valuelen);
+		if (CM_IsEmpty(msg, eCarbonCopY))
+			CM_SetField(msg, eCarbonCopY, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "Message-ID")) {
-		if (msg->cm_fields['I'] != NULL) {
+		if (!CM_IsEmpty(msg, emessageId)) {
 			syslog(LOG_WARNING, "duplicate message id\n");
 		}
+		else {
+			char *pValue;
+			long pValueLen;
 
-		if (msg->cm_fields['I'] == NULL) {
-			msg->cm_fields['I'] = strndup(value, valuelen);
-
+			pValue = value;
+			pValueLen = valuelen;
 			/* Strip angle brackets */
-			while (haschar(msg->cm_fields['I'], '<') > 0) {
-				strcpy(&msg->cm_fields['I'][0],
-					&msg->cm_fields['I'][1]);
+			while (haschar(pValue, '<') > 0) {
+				pValue ++;
+				pValueLen --;
 			}
-			for (i = 0; i<strlen(msg->cm_fields['I']); ++i)
-				if (msg->cm_fields['I'][i] == '>')
-					msg->cm_fields['I'][i] = 0;
+
+			for (i = 0; i <= pValueLen; ++i)
+				if (pValue[i] == '>') {
+					pValueLen = i;
+					break;
+				}
+
+			CM_SetField(msg, emessageId, pValue, pValueLen);
 		}
 
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "Return-Path")) {
-		if (msg->cm_fields['P'] == NULL)
-			msg->cm_fields['P'] = strndup(value, valuelen);
+		if (CM_IsEmpty(msg, eMessagePath))
+			CM_SetField(msg, eMessagePath, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "Envelope-To")) {
-		if (msg->cm_fields['V'] == NULL)
-			msg->cm_fields['V'] = strndup(value, valuelen);
+		if (CM_IsEmpty(msg, eenVelopeTo))
+			CM_SetField(msg, eenVelopeTo, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "References")) {
-		if (msg->cm_fields['W'] != NULL) {
-			free(msg->cm_fields['W']);
-		}
-		msg->cm_fields['W'] = strndup(value, valuelen);
+		CM_SetField(msg, eWeferences, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "Reply-To")) {
-		if (msg->cm_fields['K'] != NULL) {
-			free(msg->cm_fields['K']);
-		}
-		msg->cm_fields['K'] = strndup(value, valuelen);
+		CM_SetField(msg, eReplyTo, value, valuelen);
 		processed = 1;
 	}
 
 	else if (!strcasecmp(key, "In-reply-to")) {
-		if (msg->cm_fields['W'] == NULL) {		/* References: supersedes In-reply-to: */
-			msg->cm_fields['W'] = strndup(value, valuelen);
-		}
+		if (CM_IsEmpty(msg, eWeferences)) /* References: supersedes In-reply-to: */
+			CM_SetField(msg, eWeferences, value, valuelen);
 		processed = 1;
 	}
 
@@ -690,7 +1307,7 @@ int convert_field(struct CtdlMessage *msg, const char *beg, const char *end) {
 
 	/* Clean up and move on. */
 	free(key);	/* Don't free 'value', it's actually the same buffer */
-	return(processed);
+	return processed;
 }
 
 
@@ -732,7 +1349,7 @@ void convert_references_to_wefewences(char *str) {
 /*
  * Convert an RFC822 message (headers + body) to a CtdlMessage structure.
  * NOTE: the supplied buffer becomes part of the CtdlMessage structure, and
- * will be deallocated when CtdlFreeMessage() is called.  Therefore, the
+ * will be deallocated when CM_Free() is called.  Therefore, the
  * supplied buffer should be DEREFERENCED.  It should not be freed or used
  * again.
  */
@@ -749,7 +1366,6 @@ struct CtdlMessage *convert_internet_message_buf(StrBuf **rfc822)
 	struct CtdlMessage *msg;
 	const char *pos, *beg, *end, *totalend;
 	int done, alldone = 0;
-	char buf[SIZ];
 	int converted;
 	StrBuf *OtherHeaders;
 
@@ -818,21 +1434,21 @@ struct CtdlMessage *convert_internet_message_buf(StrBuf **rfc822)
 	if (pos < totalend)
 		StrBufAppendBufPlain(OtherHeaders, pos, totalend - pos, 0);
 	FreeStrBuf(rfc822);
-	msg->cm_fields['M'] = SmashStrBuf(&OtherHeaders);
+	CM_SetAsFieldSB(msg, eMesageText, &OtherHeaders);
 
 	/* Follow-up sanity checks... */
 
 	/* If there's no timestamp on this message, set it to now. */
-	if (msg->cm_fields['T'] == NULL) {
-		snprintf(buf, sizeof buf, "%ld", (long)time(NULL));
-		msg->cm_fields['T'] = strdup(buf);
+	if (CM_IsEmpty(msg, eTimestamp)) {
+		CM_SetFieldLONG(msg, eTimestamp, time(NULL));
 	}
 
 	/* If a W (references, or rather, Wefewences) field is present, we
 	 * have to convert it from RFC822 format to Citadel format.
 	 */
-	if (msg->cm_fields['W'] != NULL) {
-		convert_references_to_wefewences(msg->cm_fields['W']);
+	if (!CM_IsEmpty(msg, eWeferences)) {
+		/// todo: API!
+		convert_references_to_wefewences(msg->cm_fields[eWeferences]);
 	}
 
 	return msg;
@@ -1018,20 +1634,20 @@ char *harvest_collected_addresses(struct CtdlMessage *msg) {
 	char user[256], node[256], name[256];
 	int is_harvestable;
 	int i, j, h;
-	int field = 0;
+	eMsgField field = 0;
 
 	if (msg == NULL) return(NULL);
 
 	is_harvestable = 1;
 	strcpy(addr, "");	
-	if (msg->cm_fields['A'] != NULL) {
-		strcat(addr, msg->cm_fields['A']);
+	if (!CM_IsEmpty(msg, eAuthor)) {
+		strcat(addr, msg->cm_fields[eAuthor]);
 	}
-	if (msg->cm_fields['F'] != NULL) {
+	if (!CM_IsEmpty(msg, erFc822Addr)) {
 		strcat(addr, " <");
-		strcat(addr, msg->cm_fields['F']);
+		strcat(addr, msg->cm_fields[erFc822Addr]);
 		strcat(addr, ">");
-		if (IsDirectory(msg->cm_fields['F'], 0)) {
+		if (IsDirectory(msg->cm_fields[erFc822Addr], 0)) {
 			is_harvestable = 0;
 		}
 	}
@@ -1047,10 +1663,10 @@ char *harvest_collected_addresses(struct CtdlMessage *msg) {
 
 	/* Scan both the R (To) and Y (CC) fields */
 	for (i = 0; i < 2; ++i) {
-		if (i == 0) field = 'R' ;
-		if (i == 1) field = 'Y' ;
+		if (i == 0) field = eRecipient;
+		if (i == 1) field = eCarbonCopY;
 
-		if (msg->cm_fields[field] != NULL) {
+		if (!CM_IsEmpty(msg, field)) {
 			for (j=0; j<num_tokens(msg->cm_fields[field], ','); ++j) {
 				extract_token(addr, msg->cm_fields[field], j, ',', sizeof addr);
 				if (strstr(addr, "=?") != NULL)
