@@ -2,50 +2,16 @@
  * Message journaling functions.
  */
 
-#include "sysdep.h"
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
-#include <fcntl.h>
-
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-
-#include <ctype.h>
-#include <string.h>
-#include <limits.h>
-#include <errno.h>
-#include <stdarg.h>
-#include <sys/stat.h>
 #include <libcitadel.h>
-#include "citadel.h"
-#include "server.h"
-#include "database.h"
-#include "msgbase.h"
-#include "support.h"
-#include "sysdep_decls.h"
-#include "citserver.h"
-#include "room_ops.h"
-#include "user_ops.h"
-#include "file_ops.h"
-#include "config.h"
-#include "control.h"
-#include "genstamp.h"
-#include "internet_addressing.h"
-#include "serv_vcard.h"			/* Needed for vcard_getuser and extract_inet_email_addrs */
-#include "journaling.h"
 
 #include "ctdl_module.h"
-#include "threads.h"
+
+#include "citserver.h"
+#include "user_ops.h"
+#include "serv_vcard.h"			/* Needed for vcard_getuser and extract_inet_email_addrs */
+#include "internet_addressing.h"
+#include "journaling.h"
 
 struct jnlq *jnlq = NULL;	/* journal queue */
 
@@ -54,12 +20,12 @@ struct jnlq *jnlq = NULL;	/* journal queue */
  */
 void JournalBackgroundSubmit(struct CtdlMessage *msg,
 			StrBuf *saved_rfc822_version,
-			struct recptypes *recps) {
+			recptypes *recps) {
 
 	struct jnlq *jptr = NULL;
 
 	/* Avoid double journaling! */
-	if (msg->cm_fields['J'] != NULL) {
+	if (!CM_IsEmpty(msg, eJournal)) {
 		FreeStrBuf(&saved_rfc822_version);
 		return;
 	}
@@ -70,12 +36,12 @@ void JournalBackgroundSubmit(struct CtdlMessage *msg,
 		return;
 	}
 	memset(jptr, 0, sizeof(struct jnlq));
-	if (recps != NULL) memcpy(&jptr->recps, recps, sizeof(struct recptypes));
-	if (msg->cm_fields['A'] != NULL) jptr->from = strdup(msg->cm_fields['A']);
-	if (msg->cm_fields['N'] != NULL) jptr->node = strdup(msg->cm_fields['N']);
-	if (msg->cm_fields['F'] != NULL) jptr->rfca = strdup(msg->cm_fields['F']);
-	if (msg->cm_fields['U'] != NULL) jptr->subj = strdup(msg->cm_fields['U']);
-	if (msg->cm_fields['I'] != NULL) jptr->msgn = strdup(msg->cm_fields['I']);
+	if (recps != NULL) memcpy(&jptr->recps, recps, sizeof(recptypes));
+	if (!CM_IsEmpty(msg, eAuthor)) jptr->from = strdup(msg->cm_fields[eAuthor]);
+	if (!CM_IsEmpty(msg, eNodeName)) jptr->node = strdup(msg->cm_fields[eNodeName]);
+	if (!CM_IsEmpty(msg, erFc822Addr)) jptr->rfca = strdup(msg->cm_fields[erFc822Addr]);
+	if (!CM_IsEmpty(msg, eMsgSubject)) jptr->subj = strdup(msg->cm_fields[eMsgSubject]);
+	if (!CM_IsEmpty(msg, emessageId)) jptr->msgn = strdup(msg->cm_fields[emessageId]);
 	jptr->rfc822 = SmashStrBuf(&saved_rfc822_version);
 
 	/* Add to the queue */
@@ -122,9 +88,11 @@ void local_to_inetemail(char *inetemail, char *localuser, size_t inetemail_len)
 void JournalRunQueueMsg(struct jnlq *jmsg) {
 
 	struct CtdlMessage *journal_msg = NULL;
-	struct recptypes *journal_recps = NULL;
-	char *message_text = NULL;
+	recptypes *journal_recps = NULL;
+	StrBuf *message_text = NULL;
 	char mime_boundary[256];
+	long mblen;
+	long rfc822len;
 	char recipient[256];
 	char inetemail[256];
 	static int seq = 0;
@@ -150,14 +118,17 @@ void JournalRunQueueMsg(struct jnlq *jmsg) {
 			journal_msg->cm_magic = CTDLMESSAGE_MAGIC;
 			journal_msg->cm_anon_type = MES_NORMAL;
 			journal_msg->cm_format_type = FMT_RFC822;
-			journal_msg->cm_fields['J'] = strdup("is journal");
-			journal_msg->cm_fields['A'] = jmsg->from;
-			journal_msg->cm_fields['N'] = jmsg->node;
-			journal_msg->cm_fields['F'] = jmsg->rfca;
-			journal_msg->cm_fields['U'] = jmsg->subj;
+			CM_SetField(journal_msg, eJournal, HKEY("is journal"));
+			CM_SetField(journal_msg, eAuthor, jmsg->from, strlen(jmsg->from));
+			CM_SetField(journal_msg, eNodeName, jmsg->node, strlen(jmsg->node));
+			CM_SetField(journal_msg, erFc822Addr, jmsg->rfca, strlen(jmsg->rfca));
+			CM_SetField(journal_msg, eMsgSubject, jmsg->subj, strlen(jmsg->subj));
 
-			sprintf(mime_boundary, "--Citadel-Journal-%08lx-%04x--", time(NULL), ++seq);
-			message_text = malloc(strlen(jmsg->rfc822) + sizeof(struct recptypes) + 1024);
+			mblen = snprintf(mime_boundary, sizeof(mime_boundary),
+					 "--Citadel-Journal-%08lx-%04x--", time(NULL), ++seq);
+			rfc822len = strlen(jmsg->rfc822);
+		       
+			message_text = NewStrBufPlain(NULL, rfc822len + sizeof(recptypes) + 1024);
 
 			/*
 			 * Here is where we begin to compose the journalized message.
@@ -165,45 +136,69 @@ void JournalRunQueueMsg(struct jnlq *jmsg) {
 			 *       requested by a paying customer, and yes, it is intentionally
 			 *       spelled wrong.  Do NOT remove or change it.
 			 */
-			sprintf(message_text,
-				"Content-type: multipart/mixed; boundary=\"%s\"\r\n"
-				"Content-Identifer: ExJournalReport\r\n"
-				"MIME-Version: 1.0\r\n"
-				"\n"
-				"--%s\r\n"
-				"Content-type: text/plain\r\n"
-				"\r\n"
-				"Sender: %s "
-			,
-				mime_boundary,
-				mime_boundary,
-				( journal_msg->cm_fields['A'] ? journal_msg->cm_fields['A'] : "(null)" )
-			);
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("Content-type: multipart/mixed; boundary=\""), 0);
 
-			if (journal_msg->cm_fields['F']) {
-				sprintf(&message_text[strlen(message_text)], "<%s>",
-					journal_msg->cm_fields['F']);
-			}
-			else if (journal_msg->cm_fields['N']) {
-				sprintf(&message_text[strlen(message_text)], "@ %s",
-					journal_msg->cm_fields['N']);
-			}
+			StrBufAppendBufPlain(message_text, mime_boundary, mblen, 0);
 
-			sprintf(&message_text[strlen(message_text)],
-				"\r\n"
-				"Message-ID: <%s>\r\n"
-				"Recipients:\r\n"
-			,
-				jmsg->msgn
-			);
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("\"\r\n"
+				     "Content-Identifer: ExJournalReport\r\n"
+				     "MIME-Version: 1.0\r\n"
+				     "\n"
+				     "--"), 0);
+
+			StrBufAppendBufPlain(message_text, mime_boundary, mblen, 0);
+
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("\r\n"
+				     "Content-type: text/plain\r\n"
+				     "\r\n"
+				     "Sender: "), 0);
+
+			if (CM_IsEmpty(journal_msg, eAuthor))
+				StrBufAppendBufPlain(
+					message_text, 
+					journal_msg->cm_fields[eAuthor], -1, 0);
+			else
+				StrBufAppendBufPlain(
+					message_text, 
+					HKEY("(null)"), 0);
+
+			if (!CM_IsEmpty(journal_msg, erFc822Addr)) {
+				StrBufAppendPrintf(message_text, " <%s>",
+						   journal_msg->cm_fields[erFc822Addr]);
+			}
+			else if (!CM_IsEmpty(journal_msg, eNodeName)) {
+				StrBufAppendPrintf(message_text, " @ %s",
+						   journal_msg->cm_fields[eNodeName]);
+			}
+			else
+				StrBufAppendBufPlain(
+					message_text, 
+					HKEY(" "), 0);
+
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("\r\n"
+				     "Message-ID: <"), 0);
+
+			StrBufAppendBufPlain(message_text, jmsg->msgn, -1, 0);
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY(">\r\n"
+				     "Recipients:\r\n"), 0);
 
 			if (jmsg->recps.num_local > 0) {
 				for (i=0; i<jmsg->recps.num_local; ++i) {
 					extract_token(recipient, jmsg->recps.recp_local,
 							i, '|', sizeof recipient);
 					local_to_inetemail(inetemail, recipient, sizeof inetemail);
-					sprintf(&message_text[strlen(message_text)],
-						"	%s <%s>\r\n", recipient, inetemail);
+					StrBufAppendPrintf(message_text, 
+							   "	%s <%s>\r\n", recipient, inetemail);
 				}
 			}
 
@@ -211,8 +206,8 @@ void JournalRunQueueMsg(struct jnlq *jmsg) {
 				for (i=0; i<jmsg->recps.num_ignet; ++i) {
 					extract_token(recipient, jmsg->recps.recp_ignet,
 							i, '|', sizeof recipient);
-					sprintf(&message_text[strlen(message_text)],
-						"	%s\r\n", recipient);
+					StrBufAppendPrintf(message_text, 
+							   "	%s\r\n", recipient);
 				}
 			}
 
@@ -220,31 +215,45 @@ void JournalRunQueueMsg(struct jnlq *jmsg) {
 				for (i=0; i<jmsg->recps.num_internet; ++i) {
 					extract_token(recipient, jmsg->recps.recp_internet,
 							i, '|', sizeof recipient);
-					sprintf(&message_text[strlen(message_text)],
+					StrBufAppendPrintf(message_text, 
 						"	%s\r\n", recipient);
 				}
 			}
 
-			sprintf(&message_text[strlen(message_text)],
-				"\r\n"
-				"--%s\r\n"
-				"Content-type: message/rfc822\r\n"
-				"\r\n"
-				"%s"
-				"--%s--\r\n"
-			,
-				mime_boundary,
-				jmsg->rfc822,
-				mime_boundary
-			);
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("\r\n"
+				     "--"), 0);
 
-			journal_msg->cm_fields['M'] = message_text;
+			StrBufAppendBufPlain(message_text, mime_boundary, mblen, 0);
+
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("\r\n"
+				     "Content-type: message/rfc822\r\n"
+				     "\r\n"), 0);
+
+			StrBufAppendBufPlain(message_text, jmsg->rfc822, rfc822len, 0);
+
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("--"), 0);
+
+			StrBufAppendBufPlain(message_text, mime_boundary, mblen, 0);
+
+			StrBufAppendBufPlain(
+				message_text, 
+				HKEY("--\r\n"), 0);
+
+			CM_SetAsFieldSB(journal_msg, eMesageText, &message_text);
 			free(jmsg->rfc822);
 			free(jmsg->msgn);
+			jmsg->rfc822 = NULL;
+			jmsg->msgn = NULL;
 			
 			/* Submit journal message */
 			CtdlSubmitMsg(journal_msg, journal_recps, "", 0);
-			CtdlFreeMessage(journal_msg);
+			CM_Free(journal_msg);
 		}
 
 		free_recipients(journal_recps);
