@@ -62,8 +62,8 @@ void hunt_for_autocomplete(long msgnum, char *search_string) {
 	msg = CtdlFetchMessage(msgnum, 1);
 	if (msg == NULL) return;
 
-	v = vcard_load(msg->cm_fields['M']);
-	CtdlFreeMessage(msg);
+	v = vcard_load(msg->cm_fields[eMesageText]);
+	CM_Free(msg);
 
 	/*
 	 * Try to match from a friendly name (the "fn" field).  If there is
