@@ -30,6 +30,7 @@
 #include "msgbase.h"
 #include "internet_addressing.h"
 #include "serv_calendar.h"
+#include "room_ops.h"
 #include "euidindex.h"
 #include "ical_dezonify.h"
 
@@ -98,6 +99,7 @@ icalcomponent *ical_encapsulate_subcomponent(icalcomponent *subcomp) {
  */
 void ical_write_to_cal(struct ctdluser *u, icalcomponent *cal) {
 	char *ser = NULL;
+	long serlen;
 	icalcomponent *encaps = NULL;
 	struct CtdlMessage *msg = NULL;
 	icalcomponent *tmp=NULL;
@@ -119,13 +121,15 @@ void ical_write_to_cal(struct ctdluser *u, icalcomponent *cal) {
 	ser = icalcomponent_as_ical_string_r(cal);
 	if (ser == NULL) return;
 
+	serlen = strlen(ser);
+
 	/* If the caller supplied a user, write to that user's default calendar room */
 	if (u) {
 		/* This handy API function does all the work for us. */
 		CtdlWriteObject(USERCALENDARROOM,	/* which room */
 			"text/calendar",	/* MIME type */
 			ser,			/* data */
-			strlen(ser)+1,		/* length */
+			serlen + 1,		/* length */
 			u,			/* which user */
 			0,			/* not binary */
 			0,			/* don't delete others of this type */
@@ -135,22 +139,28 @@ void ical_write_to_cal(struct ctdluser *u, icalcomponent *cal) {
 
 	/* If the caller did not supply a user, write to the currently selected room */
 	if (!u) {
+		struct CitContext *CCC = CC;
+		StrBuf *MsgBody;
+
 		msg = malloc(sizeof(struct CtdlMessage));
 		memset(msg, 0, sizeof(struct CtdlMessage));
 		msg->cm_magic = CTDLMESSAGE_MAGIC;
 		msg->cm_anon_type = MES_NORMAL;
 		msg->cm_format_type = 4;
-		msg->cm_fields['A'] = strdup(CC->user.fullname);
-		msg->cm_fields['O'] = strdup(CC->room.QRname);
-		msg->cm_fields['N'] = strdup(config.c_nodename);
-		msg->cm_fields['H'] = strdup(config.c_humannode);
-		msg->cm_fields['M'] = malloc(strlen(ser) + 40);
-		strcpy(msg->cm_fields['M'], "Content-type: text/calendar\r\n\r\n");
-		strcat(msg->cm_fields['M'], ser);
+		CM_SetField(msg, eAuthor, CCC->user.fullname, strlen(CCC->user.fullname));
+		CM_SetField(msg, eOriginalRoom, CCC->room.QRname, strlen(CCC->room.QRname));
+		CM_SetField(msg, eNodeName, CFG_KEY(c_nodename));
+		CM_SetField(msg, eHumanNode, CFG_KEY(c_humannode));
+
+		MsgBody = NewStrBufPlain(NULL, serlen + 100);
+		StrBufAppendBufPlain(MsgBody, HKEY("Content-type: text/calendar\r\n\r\n"), 0);
+		StrBufAppendBufPlain(MsgBody, ser, serlen, 0);
+
+		CM_SetAsFieldSB(msg, eMesageText, &MsgBody);
 	
 		/* Now write the data */
 		CtdlSubmitMsg(msg, NULL, "", QP_EADDR);
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 	}
 
 	/* In either case, now we can free the serialized calendar object */
@@ -175,15 +185,15 @@ void ical_send_a_reply(icalcomponent *request, char *action) {
 	icalproperty *summary = NULL;
 	char summary_string[SIZ];
 	icalproperty *me_attend = NULL;
-	struct recptypes *recp = NULL;
+	recptypes *recp = NULL;
 	icalparameter *partstat = NULL;
 	char *serialized_reply = NULL;
 	char *reply_message_text = NULL;
 	const char *ch;
 	struct CtdlMessage *msg = NULL;
-	struct recptypes *valid = NULL;
+	recptypes *valid = NULL;
 
-	strcpy(organizer_string, "");
+	*organizer_string = '\0';
 	strcpy(summary_string, "Calendar item");
 
 	if (request == NULL) {
@@ -302,7 +312,7 @@ void ical_send_a_reply(icalcomponent *request, char *action) {
 		if (msg != NULL) {
 			valid = validate_recipients(organizer_string, NULL, 0);
 			CtdlSubmitMsg(msg, valid, "", QP_EADDR);
-			CtdlFreeMessage(msg);
+			CM_Free(msg);
 			free_recipients(valid);
 		}
 	}
@@ -376,18 +386,17 @@ void ical_respond(long msgnum, char *partnum, char *action) {
 
 	memset(&ird, 0, sizeof ird);
 	strcpy(ird.desired_partnum, partnum);
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_part,		/* callback function */
-		NULL, NULL,
-		(void *) &ird,			/* user data */
-		0
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_part,		/* callback function */
+		    NULL, NULL,
+		    (void *) &ird,			/* user data */
+		    0
 	);
 
 	/* We're done with the incoming message, because we now have a
 	 * calendar object in memory.
 	 */
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	/*
 	 * Here is the real meat of this function.  Handle the event.
@@ -632,14 +641,13 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 		return(2);			/* internal error */
 	}
 	oec.c = NULL;
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_original_event,	/* callback function */
-		NULL, NULL,
-		&oec,				/* user data */
-		0
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_original_event,	/* callback function */
+		    NULL, NULL,
+		    &oec,				/* user data */
+		    0
 	);
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	original_event = oec.c;
 	if (original_event == NULL) {
@@ -679,7 +687,7 @@ int ical_update_my_calendar_with_reply(icalcomponent *cal) {
 		if (msg != NULL) {
 			CIT_ICAL->avoid_sending_invitations = 1;
 			CtdlSubmitMsg(msg, NULL, roomname, QP_EADDR);
-			CtdlFreeMessage(msg);
+			CM_Free(msg);
 			CIT_ICAL->avoid_sending_invitations = 0;
 		}
 	}
@@ -719,18 +727,17 @@ void ical_handle_rsvp(long msgnum, char *partnum, char *action) {
 
 	memset(&ird, 0, sizeof ird);
 	strcpy(ird.desired_partnum, partnum);
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_part,		/* callback function */
-		NULL, NULL,
-		(void *) &ird,			/* user data */
-		0
-	);
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_part,		/* callback function */
+		    NULL, NULL,
+		    (void *) &ird,			/* user data */
+		    0
+		);
 
 	/* We're done with the incoming message, because we now have a
 	 * calendar object in memory.
 	 */
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	/*
 	 * Here is the real meat of this function.  Handle the event.
@@ -1151,14 +1158,13 @@ void ical_hunt_for_conflicts_backend(long msgnum, void *data) {
 	if (msg == NULL) return;
 	memset(&ird, 0, sizeof ird);
 	strcpy(ird.desired_partnum, "_HUNT_");
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_part,		/* callback function */
-		NULL, NULL,
-		(void *) &ird,			/* user data */
-		0
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_part,		/* callback function */
+		    NULL, NULL,
+		    (void *) &ird,			/* user data */
+		    0
 	);
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	if (ird.cal == NULL) return;
 
@@ -1220,15 +1226,14 @@ void ical_conflicts(long msgnum, char *partnum) {
 
 	memset(&ird, 0, sizeof ird);
 	strcpy(ird.desired_partnum, partnum);
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_part,		/* callback function */
-		NULL, NULL,
-		(void *) &ird,			/* user data */
-		0
-	);
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_part,		/* callback function */
+		    NULL, NULL,
+		    (void *) &ird,			/* user data */
+		    0
+		);
 
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	if (ird.cal != NULL) {
 		ical_hunt_for_conflicts(ird.cal);
@@ -1401,14 +1406,13 @@ void ical_freebusy_backend(long msgnum, void *data) {
 	if (msg == NULL) return;
 	memset(&ird, 0, sizeof ird);
 	strcpy(ird.desired_partnum, "_HUNT_");
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_part,		/* callback function */
-		NULL, NULL,
-		(void *) &ird,			/* user data */
-		0
-	);
-	CtdlFreeMessage(msg);
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_part,		/* callback function */
+		    NULL, NULL,
+		    (void *) &ird,			/* user data */
+		    0
+		);
+	CM_Free(msg);
 
 	if (ird.cal) {
 		ical_add_to_freebusy(fb, ird.cal);		/* Add VEVENT times to VFREEBUSY */
@@ -1429,7 +1433,7 @@ void ical_freebusy(char *who) {
 	icalcomponent *encaps = NULL;
 	icalcomponent *fb = NULL;
 	int found_user = (-1);
-	struct recptypes *recp = NULL;
+	recptypes *recp = NULL;
 	char buf[256];
 	char host[256];
 	char type[256];
@@ -1602,14 +1606,13 @@ void ical_getics_backend(long msgnum, void *data) {
 	if (msg == NULL) return;
 	memset(&ird, 0, sizeof ird);
 	strcpy(ird.desired_partnum, "_HUNT_");
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_locate_part,		/* callback function */
-		NULL, NULL,
-		(void *) &ird,			/* user data */
-		0
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_locate_part,		/* callback function */
+		    NULL, NULL,
+		    (void *) &ird,			/* user data */
+		    0
 	);
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	if (ird.cal == NULL) return;
 
@@ -1989,7 +1992,7 @@ void ical_send_out_invitations(icalcomponent *top_level_cal, icalcomponent *cal)
 	icalcomponent *encaps = NULL;
 	char *request_message_text = NULL;
 	struct CtdlMessage *msg = NULL;
-	struct recptypes *valid = NULL;
+	recptypes *valid = NULL;
 	char attendees_string[SIZ];
 	int num_attendees = 0;
 	char this_attendee[256];
@@ -2201,7 +2204,7 @@ void ical_send_out_invitations(icalcomponent *top_level_cal, icalcomponent *cal)
 		if (msg != NULL) {
 			valid = validate_recipients(attendees_string, NULL, 0);
 			CtdlSubmitMsg(msg, valid, "", QP_EADDR);
-			CtdlFreeMessage(msg);
+			CM_Free(msg);
 			free_recipients(valid);
 		}
 	}
@@ -2285,10 +2288,10 @@ void ical_obj_beforesave_backend(char *name, char *filename, char *partnum,
 		char *disp, void *content, char *cbtype, char *cbcharset, size_t length,
 		char *encoding, char *cbid, void *cbuserdata)
 {
+	const char* pch;
 	icalcomponent *cal, *nested_event, *nested_todo, *whole_cal;
 	icalproperty *p;
 	char new_uid[256] = "";
-	char buf[1024] = "";
 	struct CtdlMessage *msg = (struct CtdlMessage *) cbuserdata;
 
 	if (!msg) return;
@@ -2334,13 +2337,10 @@ void ical_obj_beforesave_backend(char *name, char *filename, char *partnum,
 				p = ical_ctdl_get_subprop(cal, ICAL_UID_PROPERTY);
 			}
 			if (p != NULL) {
-				safestrncpy(buf, icalproperty_get_comment(p), sizeof buf);
-				if (!IsEmptyStr(buf)) {
-					if (msg->cm_fields['E'] != NULL) {
-						free(msg->cm_fields['E']);
-					}
-					msg->cm_fields['E'] = strdup(buf);
-					syslog(LOG_DEBUG, "Saving calendar UID <%s>\n", buf);
+				pch = icalproperty_get_comment(p);
+				if (!IsEmptyStr(pch)) {
+					CM_SetField(msg, eExclusiveID, pch, strlen(pch));
+					syslog(LOG_DEBUG, "Saving calendar UID <%s>\n", pch);
 				}
 			}
 
@@ -2348,12 +2348,12 @@ void ical_obj_beforesave_backend(char *name, char *filename, char *partnum,
 
 			p = ical_ctdl_get_subprop(cal, ICAL_SUMMARY_PROPERTY);
 			if (p != NULL) {
-				safestrncpy(buf, icalproperty_get_comment(p), sizeof buf);
-				if (!IsEmptyStr(buf)) {
-					if (msg->cm_fields['U'] != NULL) {
-						free(msg->cm_fields['U']);
-					}
-					msg->cm_fields['U'] = rfc2047encode(buf, strlen(buf));
+				pch = icalproperty_get_comment(p);
+				if (!IsEmptyStr(pch)) {
+					char *subj;
+
+					subj = rfc2047encode(pch, strlen(pch));
+					CM_SetAsField(msg, eMsgSubject, &subj, strlen(subj));
 				}
 			}
 
@@ -2364,11 +2364,7 @@ void ical_obj_beforesave_backend(char *name, char *filename, char *partnum,
 				time_t idtstart;
 				idtstart = icaltime_as_timet(icalproperty_get_dtstart(p));
 				if (idtstart > 0) {
-					if (msg->cm_fields['T'] != NULL) {
-						free(msg->cm_fields['T']);
-					}
-					msg->cm_fields['T'] = strdup("000000000000000000");
-					sprintf(msg->cm_fields['T'], "%ld", idtstart);
+					CM_SetFieldLONG(msg, eTimestamp, idtstart);
 				}
 			}
 
@@ -2390,7 +2386,7 @@ void ical_obj_beforesave_backend(char *name, char *filename, char *partnum,
  * If the message is being saved, we also set various message header fields
  * using data found in the iCalendar object.
  */
-int ical_obj_beforesave(struct CtdlMessage *msg)
+int ical_obj_beforesave(struct CtdlMessage *msg, recptypes *recp)
 {
 	/* First determine if this is a calendar or tasks room */
 	if (  (CC->room.QRdefaultview != VIEW_CALENDAR)
@@ -2405,18 +2401,17 @@ int ical_obj_beforesave(struct CtdlMessage *msg)
 		return(1);		/* You tried to save a non-RFC822 message! */
 	}
 
-	if (msg->cm_fields['M'] == NULL) {
+	if (CM_IsEmpty(msg, eMesageText)) {
 		return(1);		/* You tried to save a null message! */
 	}
 
 	/* Do all of our lovely back-end parsing */
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_obj_beforesave_backend,
-		NULL, NULL,
-		(void *)msg,
-		0
-	);
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_obj_beforesave_backend,
+		    NULL, NULL,
+		    (void *)msg,
+		    0
+		);
 
 	return(0);
 }
@@ -2458,7 +2453,7 @@ void ical_obj_aftersave_backend(char *name, char *filename, char *partnum,
  * (This will start back end tasks such as automatic generation of invitations,
  * if such actions are appropriate.)
  */
-int ical_obj_aftersave(struct CtdlMessage *msg)
+int ical_obj_aftersave(struct CtdlMessage *msg, recptypes *recp)
 {
 	char roomname[ROOMNAMELEN];
 
@@ -2476,16 +2471,15 @@ int ical_obj_aftersave(struct CtdlMessage *msg)
 	if (msg->cm_format_type != 4) return(1);
 
 	/* Reject null messages */
-	if (msg->cm_fields['M'] == NULL) return(1);
+	if (CM_IsEmpty(msg, eMesageText)) return(1);
 	
 	/* Now recurse through it looking for our icalendar data */
-	mime_parser(msg->cm_fields['M'],
-		NULL,
-		*ical_obj_aftersave_backend,
-		NULL, NULL,
-		NULL,
-		0
-	);
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *ical_obj_aftersave_backend,
+		    NULL, NULL,
+		    NULL,
+		    0
+		);
 
 	return(0);
 }
