@@ -70,7 +70,7 @@
 /*
  * Connect to the clamd server and scan a message.
  */
-int clamd(struct CtdlMessage *msg) {
+int clamd(struct CtdlMessage *msg, recptypes *recp) {
 	int sock = (-1);
 	int streamsock = (-1);
 	char clamhosts[SIZ];
@@ -180,10 +180,7 @@ int clamd(struct CtdlMessage *msg) {
 	}
 
 	if (is_virus) {
-		if (msg->cm_fields['0'] != NULL) {
-			free(msg->cm_fields['0']);
-		}
-		msg->cm_fields['0'] = strdup("message rejected by virus filter");
+		CM_SetField(msg, eErrorMsg, HKEY("message rejected by virus filter"));
 	}
 
 bail:	close(sock);
