@@ -385,7 +385,7 @@ void init_ssl(void)
 	CtdlRegisterProtoHook(cmd_stls, "STLS", "Start SSL/TLS session");
 	CtdlRegisterProtoHook(cmd_gtls, "GTLS",
 			      "Get SSL/TLS session status");
-	CtdlRegisterSessionHook(endtls, EVT_STOP);
+	CtdlRegisterSessionHook(endtls, EVT_STOP, PRIO_STOP + 10);
 }
 
 
@@ -497,7 +497,7 @@ int client_readline_sslbuffer(StrBuf *Line, StrBuf *IOBuf, const char **Pos, int
 			StrBufAppendBufPlain(Line, pos, 
 					     StrLength(IOBuf) - (pos - ChrPtr(IOBuf)), 0);
 			FlushStrBuf(IOBuf);
-			pos = *Pos = NULL;
+			*Pos = NULL;
 		}
 		else {
 			int n = 0;
@@ -510,7 +510,7 @@ int client_readline_sslbuffer(StrBuf *Line, StrBuf *IOBuf, const char **Pos, int
 
 			if (StrLength(IOBuf) <= (pch - ChrPtr(IOBuf) + 1)) {
 				FlushStrBuf(IOBuf);
-				pos = *Pos = NULL;
+				*Pos = NULL;
 			}
 			else 
 				*Pos = pch + 1;
