@@ -36,6 +36,7 @@
 #include <curl/curl.h>
 
 #include "citadel.h"
+#include "citserver.h"
 #include "citadel_dirs.h"
 #include "clientsocket.h"
 #include "sysdep.h"
@@ -264,6 +265,7 @@ eNextState EvaluateResult(AsyncIO *IO)
 		CtdlAideMessage(ChrPtr(ErrMsg),
 				"External notifier: "
 				"unable to contact notification host!");
+		FreeStrBuf(&ErrMsg);
 	}
 
 	syslog(LOG_DEBUG, "Funambol notified\n");
@@ -300,7 +302,7 @@ eNextState EvaluateResult(AsyncIO *IO)
 	///if (SOAPMessage != NULL) free(SOAPMessage);
 	///if (buf != NULL) free(buf);
 	///FreeStrBuf (&ReplyBuf);
-	return 0;
+	return eTerminateConnection;
 }
 
 eNextState ExtNotifyTerminateDB(AsyncIO *IO)
