@@ -110,8 +110,8 @@ void imap_fetch_internaldate(struct CtdlMessage *msg) {
 	time_t msgdate;
 
 	if (!msg) return;
-	if (msg->cm_fields['T'] != NULL) {
-		msgdate = atol(msg->cm_fields['T']);
+	if (!CM_IsEmpty(msg, eTimestamp)) {
+		msgdate = atol(msg->cm_fields[eTimestamp]);
 	}
 	else {
 		msgdate = time(NULL);
@@ -190,7 +190,7 @@ void imap_fetch_rfc822(long msgnum, const char *whichfmt) {
 		CCC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
 		CtdlOutputMsg(msgnum, MT_RFC822,
 			(need_body ? HEADERS_ALL : HEADERS_FAST),
-			0, 1, NULL, SUPPRESS_ENV_TO, NULL, NULL
+			0, 1, NULL, SUPPRESS_ENV_TO, NULL, NULL, NULL
 		);
 		if (!need_body) IAPuts("\r\n");	/* extra trailing newline */
 		Imap->cached_rfc822 = CCC->redirect_buffer;
@@ -355,25 +355,25 @@ void imap_output_envelope_from(struct CtdlMessage *msg) {
 
 	/* For everything else, we do stuff. */
 	IAPuts("(("); /* open double-parens */
-	plain_imap_strout(msg->cm_fields['A']);	/* personal name */
+	IPutMsgField(eAuthor);	/* personal name */
 	IAPuts(" NIL ");	/* source route (not used) */
 
 
-	if (msg->cm_fields['F'] != NULL) {
-		process_rfc822_addr(msg->cm_fields['F'], user, node, name);
-		plain_imap_strout(user);		/* mailbox name (user id) */
+	if (!CM_IsEmpty(msg, erFc822Addr)) {
+		process_rfc822_addr(msg->cm_fields[erFc822Addr], user, node, name);
+		IPutStr(user, strlen(user));		/* mailbox name (user id) */
 		IAPuts(" ");
 		if (!strcasecmp(node, config.c_nodename)) {
-			plain_imap_strout(config.c_fqdn);
+			IPutStr(CFG_KEY(c_fqdn));
 		}
 		else {
-			plain_imap_strout(node);		/* host name */
+			IPutStr(node, strlen(node));		/* host name */
 		}
 	}
 	else {
-		plain_imap_strout(msg->cm_fields['A']); /* mailbox name (user id) */
+		IPutMsgField(eAuthor); /* mailbox name (user id) */
 		IAPuts(" ");
-		plain_imap_strout(msg->cm_fields['N']);	/* host name */
+		IPutMsgField(eNodeName);	/* host name */
 	}
 	
 	IAPuts(")) "); /* close double-parens */
@@ -416,11 +416,11 @@ void imap_output_envelope_addr(char *addr) {
 		striplt(individual_addr);
 		process_rfc822_addr(individual_addr, user, node, name);
 		IAPuts("(");
-		plain_imap_strout(name);
+		IPutStr(name, strlen(name));
 		IAPuts(" NIL ");
-		plain_imap_strout(user);
+		IPutStr(user, strlen(user));
 		IAPuts(" ");
-		plain_imap_strout(node);
+		IPutStr(node, strlen(node));
 		IAPuts(")");
 		if (i < (num_addrs-1)) 
 			IAPuts(" ");
@@ -445,14 +445,14 @@ void imap_fetch_envelope(struct CtdlMessage *msg) {
 	if (!msg) return;
 
 	/* Parse the message date into an IMAP-format date string */
-	if (msg->cm_fields['T'] != NULL) {
-		msgdate = atol(msg->cm_fields['T']);
+	if (!CM_IsEmpty(msg, eTimestamp)) {
+		msgdate = atol(msg->cm_fields[eTimestamp]);
 	}
 	else {
 		msgdate = time(NULL);
 	}
-	datestring(datestringbuf, sizeof datestringbuf,
-		msgdate, DATESTRING_IMAP);
+	len = datestring(datestringbuf, sizeof datestringbuf,
+			 msgdate, DATESTRING_IMAP);
 
 	/* Now start spewing data fields.  The order is important, as it is
 	 * defined by the protocol specification.  Nonexistent fields must
@@ -462,18 +462,18 @@ void imap_fetch_envelope(struct CtdlMessage *msg) {
 	IAPuts("ENVELOPE (");
 
 	/* Date */
-	plain_imap_strout(datestringbuf);
+	IPutStr(datestringbuf, len);
 	IAPuts(" ");
 
 	/* Subject */
-	plain_imap_strout(msg->cm_fields['U']);
+	IPutMsgField(eMsgSubject);
 	IAPuts(" ");
 
 	/* From */
 	imap_output_envelope_from(msg);
 
 	/* Sender (default to same as 'From' if not present) */
-	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Sender");
+	fieldptr = rfc822_fetch_field(msg->cm_fields[eMesageText], "Sender");
 	if (fieldptr != NULL) {
 		imap_output_envelope_addr(fieldptr);
 		free(fieldptr);
@@ -483,7 +483,7 @@ void imap_fetch_envelope(struct CtdlMessage *msg) {
 	}
 
 	/* Reply-to */
-	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Reply-to");
+	fieldptr = rfc822_fetch_field(msg->cm_fields[eMesageText], "Reply-to");
 	if (fieldptr != NULL) {
 		imap_output_envelope_addr(fieldptr);
 		free(fieldptr);
@@ -493,53 +493,53 @@ void imap_fetch_envelope(struct CtdlMessage *msg) {
 	}
 
 	/* To */
-	imap_output_envelope_addr(msg->cm_fields['R']);
+	imap_output_envelope_addr(msg->cm_fields[eRecipient]);
 
 	/* Cc (we do it this way because there might be a legacy non-Citadel Cc: field present) */
-	fieldptr = msg->cm_fields['Y'];
+	fieldptr = msg->cm_fields[eCarbonCopY];
 	if (fieldptr != NULL) {
 		imap_output_envelope_addr(fieldptr);
 	}
 	else {
-		fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Cc");
+		fieldptr = rfc822_fetch_field(msg->cm_fields[eMesageText], "Cc");
 		imap_output_envelope_addr(fieldptr);
 		if (fieldptr != NULL) free(fieldptr);
 	}
 
 	/* Bcc */
-	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "Bcc");
+	fieldptr = rfc822_fetch_field(msg->cm_fields[eMesageText], "Bcc");
 	imap_output_envelope_addr(fieldptr);
 	if (fieldptr != NULL) free(fieldptr);
 
 	/* In-reply-to */
-	fieldptr = rfc822_fetch_field(msg->cm_fields['M'], "In-reply-to");
-	plain_imap_strout(fieldptr);
+	fieldptr = rfc822_fetch_field(msg->cm_fields[eMesageText], "In-reply-to");
+	IPutStr(fieldptr, (fieldptr)?strlen(fieldptr):0);
 	IAPuts(" ");
 	if (fieldptr != NULL) free(fieldptr);
 
 	/* message ID */
-	len = strlen(msg->cm_fields['I']);
+	len = msg->cm_lengths[emessageId];
 	
 	if ((len == 0) || (
-		    (msg->cm_fields['I'][0] == '<') && 
-		    (msg->cm_fields['I'][len - 1] == '>'))
+		    (msg->cm_fields[emessageId][0] == '<') && 
+		    (msg->cm_fields[emessageId][len - 1] == '>'))
 		)
 	{
-		plain_imap_strout(msg->cm_fields['I']);
+		IPutMsgField(emessageId);
 	}
 	else 
 	{
 		char *Buf = malloc(len + 3);
 		long pos = 0;
 		
-		if (msg->cm_fields['I'][0] != '<')
+		if (msg->cm_fields[emessageId][0] != '<')
 		{
 			Buf[pos] = '<';
 			pos ++;
 		}
-		memcpy(&Buf[pos], msg->cm_fields['I'], len);
+		memcpy(&Buf[pos], msg->cm_fields[emessageId], len);
 		pos += len;
-		if (msg->cm_fields['I'][len] != '>')
+		if (msg->cm_fields[emessageId][len] != '>')
 		{
 			Buf[pos] = '>';
 			pos++;
@@ -752,7 +752,7 @@ void imap_fetch_body(long msgnum, ConstStr item, int is_peek) {
 	 * (Note value of 1 passed as 'dont_decode' so client gets it encoded)
 	 */
 	else {
-		mime_parser(msg->cm_fields['M'], NULL,
+		mime_parser(CM_RANGE(msg, eMesageText),
 			    *imap_load_part, NULL, NULL,
 			    section,
 			    1
@@ -790,7 +790,7 @@ void imap_fetch_body(long msgnum, ConstStr item, int is_peek) {
 	iaputs(&Imap->cached_body[pstart], pbytes);
 
 	if (msg != NULL) {
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 	}
 
 	/* Mark this message as "seen" *unless* this is a "peek" operation */
@@ -822,14 +822,14 @@ void imap_fetch_bodystructure_post(
 		void *content, char *cbtype, char *cbcharset, size_t length, char *encoding,
 		char *cbid, void *cbuserdata
 		) {
-
+	long len;
 	char subtype[128];
 
 	IAPuts(" ");
 
 	/* disposition */
-	extract_token(subtype, cbtype, 1, '/', sizeof subtype);
-	plain_imap_strout(subtype);
+	len = extract_token(subtype, cbtype, 1, '/', sizeof subtype);
+	IPutStr(subtype, len);
 
 	/* body language */
 	/* IAPuts(" NIL"); We thought we needed this at one point, but maybe we don't... */
@@ -855,56 +855,57 @@ void imap_fetch_bodystructure_part(
 	size_t i;
 	char cbmaintype[128];
 	char cbsubtype[128];
+	long cbmaintype_len;
+	long cbsubtype_len;
 
 	if (cbtype != NULL) if (!IsEmptyStr(cbtype)) have_cbtype = 1;
 	if (have_cbtype) {
-		extract_token(cbmaintype, cbtype, 0, '/', sizeof cbmaintype);
-		extract_token(cbsubtype, cbtype, 1, '/', sizeof cbsubtype);
+		cbmaintype_len = extract_token(cbmaintype, cbtype, 0, '/', sizeof cbmaintype);
+		cbsubtype_len = extract_token(cbsubtype, cbtype, 1, '/', sizeof cbsubtype);
 	}
 	else {
 		strcpy(cbmaintype, "TEXT");
+		cbmaintype_len = 4;
 		strcpy(cbsubtype, "PLAIN");
+		cbsubtype_len = 5;
 	}
 
 	IAPuts("(");
-	plain_imap_strout(cbmaintype);					/* body type */
+	IPutStr(cbmaintype, cbmaintype_len);			/* body type */
 	IAPuts(" ");
-	plain_imap_strout(cbsubtype);						/* body subtype */
+	IPutStr(cbsubtype, cbsubtype_len);			/* body subtype */
 	IAPuts(" ");
 
-	IAPuts("(");							/* begin body parameter list */
+	IAPuts("(");						/* begin body parameter list */
 
 	/* "NAME" must appear as the first parameter.  This is not required by IMAP,
 	 * but the Asterisk voicemail application blindly assumes that NAME will be in
 	 * the first position.  If it isn't, it rejects the message.
 	 */
-	if (name != NULL) if (!IsEmptyStr(name)) {
+	if ((name != NULL) && (!IsEmptyStr(name))) {
 		IAPuts("\"NAME\" ");
-		plain_imap_strout(name);
+		IPutStr(name, strlen(name));
 		IAPuts(" ");
 	}
 
 	IAPuts("\"CHARSET\" ");
-	if (cbcharset == NULL) {
-		plain_imap_strout("US-ASCII");
-	}
-	else if (cbcharset[0] == 0) {
-		plain_imap_strout("US-ASCII");
+	if ((cbcharset == NULL) || (cbcharset[0] == 0)){
+		IPutStr(HKEY("US-ASCII"));
 	}
 	else {
-		plain_imap_strout(cbcharset);
+		IPutStr(cbcharset, strlen(cbcharset));
 	}
-	IAPuts(") ");							/* end body parameter list */
+	IAPuts(") ");						/* end body parameter list */
 
 	IAPuts("NIL ");						/* Body ID */
 	IAPuts("NIL ");						/* Body description */
 
-	if (encoding != NULL) if (encoding[0] != 0)  have_encoding = 1;
+	if ((encoding != NULL) && (encoding[0] != 0))  have_encoding = 1;
 	if (have_encoding) {
-		plain_imap_strout(encoding);
+		IPutStr(encoding, strlen(encoding));
 	}
 	else {
-		plain_imap_strout("7BIT");
+		IPutStr(HKEY("7BIT"));
 	}
 	IAPuts(" ");
 
@@ -935,18 +936,15 @@ void imap_fetch_bodystructure_part(
 	IAPuts("NIL ");
 
 	/* Disposition */
-	if (disp == NULL) {
-		IAPuts("NIL");
-	}
-	else if (IsEmptyStr(disp)) {
+	if ((disp == NULL) || IsEmptyStr(disp)) {
 		IAPuts("NIL");
 	}
 	else {
 		IAPuts("(");
-		plain_imap_strout(disp);
-		if (filename != NULL) if (!IsEmptyStr(filename)) {
+		IPutStr(disp, strlen(disp));
+		if ((filename != NULL) && (!IsEmptyStr(filename))) {
 			IAPuts(" (\"FILENAME\" ");
-			plain_imap_strout(filename);
+			IPutStr(filename, strlen(filename));
 			IAPuts(")");
 		}
 		IAPuts(")");
@@ -1019,13 +1017,12 @@ void imap_fetch_bodystructure (long msgnum, const char *item,
 
 	/* For messages already stored in RFC822 format, we have to parse. */
 	IAPuts("BODYSTRUCTURE ");
-	mime_parser(msg->cm_fields['M'],
-			NULL,
-			*imap_fetch_bodystructure_part,	/* part */
-			*imap_fetch_bodystructure_pre,	/* pre-multi */
-			*imap_fetch_bodystructure_post,	/* post-multi */
-			NULL,
-			1);	/* don't decode -- we want it as-is */
+	mime_parser(CM_RANGE(msg, eMesageText),
+		    *imap_fetch_bodystructure_part,	/* part */
+		    *imap_fetch_bodystructure_pre,	/* pre-multi */
+		    *imap_fetch_bodystructure_post,	/* post-multi */
+		    NULL,
+		    1);	/* don't decode -- we want it as-is */
 }
 
 
@@ -1084,7 +1081,7 @@ void imap_do_fetch_msg(int seq, citimap_command *Cmd) {
 		 */
 		else if (!strcasecmp(Cmd->Params[i].Key, "BODYSTRUCTURE")) {
 			if ((msg != NULL) && (!body_loaded)) {
-				CtdlFreeMessage(msg);	/* need the whole thing */
+				CM_Free(msg);	/* need the whole thing */
 				msg = NULL;
 			}
 			if (msg == NULL) {
@@ -1115,7 +1112,7 @@ void imap_do_fetch_msg(int seq, citimap_command *Cmd) {
 	IAPuts(")\r\n");
 	unbuffer_output();
 	if (msg != NULL) {
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 	}
 }
 
