@@ -276,10 +276,10 @@ void cmd_mgsve_auth(int num_parms, char **parms, struct sdm_userdata *u)
 			message = ReadString(GetSizeToken(parms[2]), parms[0]);
 		
 		if (message != NULL) {/**< do we have tokenized login? */
-			CtdlDecodeBase64(auth, MGSVE->transmitted_message, SIZ);
+		        CtdlDecodeBase64(auth, MGSVE->transmitted_message, strlen(MGSVE->transmitted_message));
 		}
 		else 
-			CtdlDecodeBase64(auth, parms[2], SIZ);
+		        CtdlDecodeBase64(auth, parms[2], strlen(parms[2]));
 		username = auth;
 		if ((*username == '\0') && (*(username + 1) != '\0'))
 			username ++;
@@ -507,7 +507,10 @@ void mgsve_auth(char *argbuf) {
 		if (strlen(argbuf) >= 7) {
 		}
 		else {
-			CtdlEncodeBase64(username_prompt, "Username:", 9, 0);
+			size_t len = CtdlEncodeBase64(username_prompt, "Username:", 9, 0);
+			if (username_prompt[len - 1] == '\n') {
+				username_prompt[len - 1] = '\0';
+			}
 			cprintf("334 %s\r\n", username_prompt);
 		}
 		return;
