@@ -36,6 +36,7 @@ struct SpoolControl {
 	StrBuf *RoomInfo;
 	StrBuf *ListID;
 	FILE *digestfp;
+	int haveDigest;
 	int num_msgs_spooled;
 	long lastsent;
 
@@ -55,7 +56,7 @@ void InspectQueuedRoom(SpoolControl **pSC,
 
 int HaveSpoolConfig(OneRoomNetCfg* RNCfg);
 
-
+void Netmap_AddMe(struct CtdlMessage *msg, const char *defl, long defllen);
 void network_do_spoolin(HashList *working_ignetcfg, HashList *the_netmap, int *netmap_changed);
 void network_consolidate_spoolout(HashList *working_ignetcfg, HashList *the_netmap);
 void free_spoolcontrol_struct(SpoolControl **scc);
