@@ -72,7 +72,6 @@
 #include "internet_addressing.h"
 #include "serv_network.h"
 #include "clientsocket.h"
-#include "file_ops.h"
 #include "citadel_dirs.h"
 #include "threads.h"
 #include "context.h"
@@ -133,12 +132,11 @@ static void ListCalculateSubject(struct CtdlMessage *msg)
 	int rlen;
 	char *pCh;
 
-	if (msg->cm_fields['U'] == NULL) {
+	if (CM_IsEmpty(msg, eMsgSubject)) {
 		Subject = NewStrBufPlain(HKEY("(no subject)"));
 	}
 	else {
-		Subject = NewStrBufPlain(
-			msg->cm_fields['U'], -1);
+		Subject = NewStrBufPlain(CM_KEY(msg, eMsgSubject));
 	}
 	FlatSubject = NewStrBufPlain(NULL, StrLength(Subject));
 	StrBuf_RFC822_to_Utf8(FlatSubject, Subject, NULL, NULL);
@@ -165,9 +163,7 @@ static void ListCalculateSubject(struct CtdlMessage *msg)
 		StrBufRFC2047encode(&Subject, FlatSubject);
 	}
 
-	if (msg->cm_fields['U'] != NULL)
-		free (msg->cm_fields['U']);
-	msg->cm_fields['U'] = SmashStrBuf(&Subject);
+	CM_SetAsFieldSB(msg, eMsgSubject, &Subject);
 
 	FreeStrBuf(&FlatSubject);
 }
@@ -177,38 +173,34 @@ static void ListCalculateSubject(struct CtdlMessage *msg)
  */
 void network_deliver_digest(SpoolControl *sc)
 {
+	struct CitContext *CCC = CC;
+	long len;
 	char buf[SIZ];
+	char *pbuf;
 	struct CtdlMessage *msg = NULL;
 	long msglen;
-	struct recptypes *valid;
+	recptypes *valid;
 	char bounce_to[256];
 
-	if (sc->Users[listrecp] == NULL)
+	if (sc->Users[digestrecp] == NULL)
 		return;
 
-	if (sc->num_msgs_spooled < 1) {
-		fclose(sc->digestfp);
-		sc->digestfp = NULL;
-		return;
-	}
-
 	msg = malloc(sizeof(struct CtdlMessage));
 	memset(msg, 0, sizeof(struct CtdlMessage));
 	msg->cm_magic = CTDLMESSAGE_MAGIC;
 	msg->cm_format_type = FMT_RFC822;
 	msg->cm_anon_type = MES_NORMAL;
 
-	sprintf(buf, "%ld", time(NULL));
-	msg->cm_fields['T'] = strdup(buf);
-	msg->cm_fields['A'] = strdup(CC->room.QRname);
-	snprintf(buf, sizeof buf, "[%s]", CC->room.QRname);
-	msg->cm_fields['U'] = strdup(buf);
+	CM_SetFieldLONG(msg, eTimestamp, time(NULL));
+	CM_SetField(msg, eAuthor, CCC->room.QRname, strlen(CCC->room.QRname));
+	len = snprintf(buf, sizeof buf, "[%s]", CCC->room.QRname);
+	CM_SetField(msg, eMsgSubject, buf, len);
 
-	CtdlMsgSetCM_Fields(msg, 'F', SKEY(sc->Users[roommailalias]));
-	CtdlMsgSetCM_Fields(msg, 'R', SKEY(sc->Users[roommailalias]));
+	CM_SetField(msg, erFc822Addr, SKEY(sc->Users[roommailalias]));
+	CM_SetField(msg, eRecipient, SKEY(sc->Users[roommailalias]));
 
 	/* Set the 'List-ID' header */
-	CtdlMsgSetCM_Fields(msg, 'L', SKEY(sc->ListID));
+	CM_SetField(msg, eListID, SKEY(sc->ListID));
 
 	/*
 	 * Go fetch the contents of the digest
@@ -216,30 +208,26 @@ void network_deliver_digest(SpoolControl *sc)
 	fseek(sc->digestfp, 0L, SEEK_END);
 	msglen = ftell(sc->digestfp);
 
-	msg->cm_fields['M'] = malloc(msglen + 1);
+	pbuf = malloc(msglen + 1);
 	fseek(sc->digestfp, 0L, SEEK_SET);
-	fread(msg->cm_fields['M'], (size_t)msglen, 1, sc->digestfp);
-	msg->cm_fields['M'][msglen] = '\0';
-
-	fclose(sc->digestfp);
-	sc->digestfp = NULL;
+	fread(pbuf, (size_t)msglen, 1, sc->digestfp);
+	pbuf[msglen] = '\0';
+	CM_SetAsField(msg, eMesageText, &pbuf, msglen);
 
 	/* Now generate the delivery instructions */
-	if (sc->Users[listrecp] == NULL)
-		return;
 
 	/* Where do we want bounces and other noise to be heard?
-	 *Surely not the list members! */
+	 * Surely not the list members! */
 	snprintf(bounce_to, sizeof bounce_to, "room_aide@%s", config.c_fqdn);
 
 	/* Now submit the message */
-	valid = validate_recipients(ChrPtr(sc->Users[listrecp]), NULL, 0);
+	valid = validate_recipients(ChrPtr(sc->Users[digestrecp]), NULL, 0);
 	if (valid != NULL) {
 		valid->bounce_to = strdup(bounce_to);
 		valid->envelope_from = strdup(bounce_to);
 		CtdlSubmitMsg(msg, valid, NULL, 0);
 	}
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 	free_recipients(valid);
 }
 
@@ -249,40 +237,52 @@ void network_process_digest(SpoolControl *sc, struct CtdlMessage *omsg, long *de
 
 	struct CtdlMessage *msg = NULL;
 
-	/*
-	 * Process digest recipients
-	 */
-	if ((sc->Users[digestrecp] == NULL)||
-	    (sc->digestfp == NULL))
+	if (sc->Users[digestrecp] == NULL)
 		return;
 
-	msg = CtdlDuplicateMessage(omsg);
+	/* If there are digest recipients, we have to build a digest */
+	if (sc->digestfp == NULL) {
+		
+		sc->digestfp = create_digest_file(&sc->room, 1);
+
+		if (sc->digestfp == NULL)
+			return;
+
+		sc->haveDigest = ftell(sc->digestfp) > 0;
+		if (!sc->haveDigest) {
+			fprintf(sc->digestfp, "Content-type: text/plain\n\n");
+		}
+		sc->haveDigest = 1;
+	}
+
+	msg = CM_Duplicate(omsg);
 	if (msg != NULL) {
+		sc->haveDigest = 1;
 		fprintf(sc->digestfp,
 			" -----------------------------------"
 			"------------------------------------"
 			"-------\n");
 		fprintf(sc->digestfp, "From: ");
-		if (msg->cm_fields['A'] != NULL) {
+		if (!CM_IsEmpty(msg, eAuthor)) {
 			fprintf(sc->digestfp,
 				"%s ",
-				msg->cm_fields['A']);
+				msg->cm_fields[eAuthor]);
 		}
-		if (msg->cm_fields['F'] != NULL) {
+		if (!CM_IsEmpty(msg, erFc822Addr)) {
 			fprintf(sc->digestfp,
 				"<%s> ",
-				msg->cm_fields['F']);
+				msg->cm_fields[erFc822Addr]);
 		}
-		else if (msg->cm_fields['N'] != NULL) {
+		else if (!CM_IsEmpty(msg, eNodeName)) {
 			fprintf(sc->digestfp,
 				"@%s ",
-				msg->cm_fields['N']);
+				msg->cm_fields[eNodeName]);
 		}
 		fprintf(sc->digestfp, "\n");
-		if (msg->cm_fields['U'] != NULL) {
+		if (!CM_IsEmpty(msg, eMsgSubject)) {
 			fprintf(sc->digestfp,
 				"Subject: %s\n",
-				msg->cm_fields['U']);
+				msg->cm_fields[eMsgSubject]);
 		}
 
 		CC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
@@ -304,7 +304,7 @@ void network_process_digest(SpoolControl *sc, struct CtdlMessage *omsg, long *de
 		FreeStrBuf(&CC->redirect_buffer);
 
 		sc->num_msgs_spooled += 1;
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 	}
 }
 
@@ -324,23 +324,22 @@ void network_process_list(SpoolControl *sc, struct CtdlMessage *omsg, long *dele
 	 * in order to insert the [list name] in it, etc.
 	 */
 
-	msg = CtdlDuplicateMessage(omsg);
+	msg = CM_Duplicate(omsg);
 
 
-	CtdlMsgSetCM_Fields(msg, 'K', SKEY(sc->Users[roommailalias]));
+	CM_SetField(msg, eReplyTo, SKEY(sc->Users[roommailalias]));
 
 	/* if there is no other recipient, Set the recipient
 	 * of the list message to the email address of the
 	 * room itself.
 	 */
-	if ((msg->cm_fields['R'] == NULL) ||
-	    IsEmptyStr(msg->cm_fields['R']))
+	if (CM_IsEmpty(msg, eRecipient))
 	{
-		CtdlMsgSetCM_Fields(msg, 'R', SKEY(sc->Users[roommailalias]));
+		CM_SetField(msg, eRecipient, SKEY(sc->Users[roommailalias]));
 	}
 
 	/* Set the 'List-ID' header */
-	CtdlMsgSetCM_Fields(msg, 'L', SKEY(sc->ListID));
+	CM_SetField(msg, eListID, SKEY(sc->ListID));
 
 
 	/* Prepend "[List name]" to the subject */
@@ -348,7 +347,7 @@ void network_process_list(SpoolControl *sc, struct CtdlMessage *omsg, long *dele
 
 	/* Handle delivery */
 	network_deliver_list(msg, sc, CC->room.QRname);
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 }
 
 /*
@@ -356,7 +355,7 @@ void network_process_list(SpoolControl *sc, struct CtdlMessage *omsg, long *dele
  */
 void network_deliver_list(struct CtdlMessage *msg, SpoolControl *sc, const char *RoomName)
 {
-	struct recptypes *valid;
+	recptypes *valid;
 	char bounce_to[256];
 
 	/* Don't do this if there were no recipients! */
@@ -378,7 +377,7 @@ void network_deliver_list(struct CtdlMessage *msg, SpoolControl *sc, const char
 		CtdlSubmitMsg(msg, valid, NULL, 0);
 		free_recipients(valid);
 	}
-	/* Do not call CtdlFreeMessage(msg) here; the caller will free it. */
+	/* Do not call CM_Free(msg) here; the caller will free it. */
 }
 
 
@@ -387,7 +386,7 @@ void network_process_participate(SpoolControl *sc, struct CtdlMessage *omsg, lon
 	struct CtdlMessage *msg = NULL;
 	int ok_to_participate = 0;
 	StrBuf *Buf = NULL;
-	struct recptypes *valid;
+	recptypes *valid;
 
 	/*
 	 * Process client-side list participations for this room
@@ -395,7 +394,7 @@ void network_process_participate(SpoolControl *sc, struct CtdlMessage *omsg, lon
 	if (sc->Users[participate] == NULL)
 		return;
 
-	msg = CtdlDuplicateMessage(omsg);
+	msg = CM_Duplicate(omsg);
 
 	/* Only send messages which originated on our own
 	 * Citadel network, otherwise we'll end up sending the
@@ -403,13 +402,13 @@ void network_process_participate(SpoolControl *sc, struct CtdlMessage *omsg, lon
 	 * is rude...
 	 */
 	ok_to_participate = 0;
-	if (msg->cm_fields['N'] != NULL) {
-		if (!strcasecmp(msg->cm_fields['N'],
+	if (!CM_IsEmpty(msg, eNodeName)) {
+		if (!strcasecmp(msg->cm_fields[eNodeName],
 				config.c_nodename)) {
 			ok_to_participate = 1;
 		}
 		
-		Buf = NewStrBufPlain(msg->cm_fields['N'], -1);
+		Buf = NewStrBufPlain(CM_KEY(msg, eNodeName));
 		if (CtdlIsValidNode(NULL,
 				    NULL,
 				    Buf,
@@ -426,16 +425,16 @@ void network_process_participate(SpoolControl *sc, struct CtdlMessage *omsg, lon
 		 * room itself, so the remote listserv doesn't
 		 * reject us.
 		 */
-		CtdlMsgSetCM_Fields(msg, 'F', SKEY(sc->Users[roommailalias]));
+		CM_SetField(msg, erFc822Addr, SKEY(sc->Users[roommailalias]));
 
 		valid = validate_recipients(ChrPtr(sc->Users[participate]) , NULL, 0);
 
-		CtdlMsgSetCM_Fields(msg, 'R', SKEY(sc->Users[roommailalias]));
+		CM_SetField(msg, eRecipient, SKEY(sc->Users[roommailalias]));
 		CtdlSubmitMsg(msg, valid, "", 0);
 		free_recipients(valid);
 	}
 	FreeStrBuf(&Buf);
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 }
 
 void network_process_ignetpush(SpoolControl *sc, struct CtdlMessage *omsg, long *delete_after_send)
@@ -449,8 +448,6 @@ void network_process_ignetpush(SpoolControl *sc, struct CtdlMessage *omsg, long
 	char buf[SIZ];
 	char filename[PATH_MAX];
 	FILE *fp;
-	size_t newpath_len;
-	char *newpath = NULL;
 	StrBuf *Buf = NULL;
 	int i;
 	int bang = 0;
@@ -462,29 +459,19 @@ void network_process_ignetpush(SpoolControl *sc, struct CtdlMessage *omsg, long
 	/*
 	 * Process IGnet push shares
 	 */
-	msg = CtdlDuplicateMessage(omsg);
+	msg = CM_Duplicate(omsg);
 
 	/* Prepend our node name to the Path field whenever
 	 * sending a message to another IGnet node
 	 */
-	if (msg->cm_fields['P'] == NULL)
-	{
-		msg->cm_fields['P'] = strdup("username");
-	}
-	newpath_len = strlen(msg->cm_fields['P']) +
-		strlen(config.c_nodename) + 2;
-	newpath = malloc(newpath_len);
-	snprintf(newpath, newpath_len, "%s!%s",
-		 config.c_nodename, msg->cm_fields['P']);
-	free(msg->cm_fields['P']);
-	msg->cm_fields['P'] = newpath;
-	
+	Netmap_AddMe(msg, HKEY("username"));
+
 	/*
 	 * Determine if this message is set to be deleted
 	 * after sending out on the network
 	 */
-	if (msg->cm_fields['S'] != NULL) {
-		if (!strcasecmp(msg->cm_fields['S'], "CANCEL")) {
+	if (!CM_IsEmpty(msg, eSpecialField)) {
+		if (!strcasecmp(msg->cm_fields[eSpecialField], "CANCEL")) {
 			*delete_after_send = 1;
 		}
 	}
@@ -514,12 +501,12 @@ void network_process_ignetpush(SpoolControl *sc, struct CtdlMessage *omsg, long
 		}
 		
 		/* Check for split horizon */
-		QN_syslog(LOG_DEBUG, "Path is %s\n", msg->cm_fields['P']);
-		bang = num_tokens(msg->cm_fields['P'], '!');
+		QN_syslog(LOG_DEBUG, "Path is %s\n", msg->cm_fields[eMessagePath]);
+		bang = num_tokens(msg->cm_fields[eMessagePath], '!');
 		if (bang > 1) {
 			for (i=0; i<(bang-1); ++i) {
 				extract_token(buf,
-					      msg->cm_fields['P'],
+					      msg->cm_fields[eMessagePath],
 					      i, '!',
 					      sizeof buf);
 
@@ -545,20 +532,15 @@ void network_process_ignetpush(SpoolControl *sc, struct CtdlMessage *omsg, long
 			 * room on the far end by setting the C field
 			 * correctly
 			 */
-			if (msg->cm_fields['C'] != NULL) {
-				free(msg->cm_fields['C']);
-			}
 			if (StrLength(RemoteRoom) > 0) {
-				msg->cm_fields['C'] =
-					strdup(ChrPtr(RemoteRoom));
+				CM_SetField(msg, eRemoteRoom, SKEY(RemoteRoom));
 			}
 			else {
-				msg->cm_fields['C'] =
-					strdup(CC->room.QRname);
+				CM_SetField(msg, eRemoteRoom, CCC->room.QRname, strlen(CCC->room.QRname));
 			}
 			
 			/* serialize it for transmission */
-			serialize_message(&sermsg, msg);
+			CtdlSerializeMessage(&sermsg, msg);
 			if (sermsg.len > 0) {
 				
 				/* write it to a spool file */
@@ -596,7 +578,7 @@ void network_process_ignetpush(SpoolControl *sc, struct CtdlMessage *omsg, long
 	FreeStrBuf(&Buf);
 	FreeStrBuf(&Recipient);
 	FreeStrBuf(&RemoteRoom);
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 }
 
 
@@ -627,7 +609,7 @@ void network_spool_msg(long msgnum,
 	network_process_participate(sc, msg, &delete_after_send);
 	network_process_ignetpush(sc, msg, &delete_after_send);
 	
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	/* update lastsent */
 	sc->lastsent = msgnum;
