@@ -72,7 +72,6 @@
 #include "internet_addressing.h"
 #include "serv_network.h"
 #include "clientsocket.h"
-#include "file_ops.h"
 #include "citadel_dirs.h"
 #include "threads.h"
 #include "context.h"
@@ -160,13 +159,30 @@ int HaveSpoolConfig(OneRoomNetCfg* RNCfg)
 	return interested;
 }
 
+void Netmap_AddMe(struct CtdlMessage *msg, const char *defl, long defllen)
+{
+	long node_len;
+	char buf[SIZ];
 
+	/* prepend our node to the path */
+	if (CM_IsEmpty(msg, eMessagePath)) {
+		CM_SetField(msg, eMessagePath, defl, defllen);
+	}
+	node_len = configlen.c_nodename;
+	if (node_len >= SIZ) 
+		node_len = SIZ - 1;
+	memcpy(buf, config.c_nodename, node_len);
+	buf[node_len] = '!';
+	buf[node_len + 1] = '\0';
+	CM_PrependToField(msg, eMessagePath, buf, node_len + 1);
+}
 
 void InspectQueuedRoom(SpoolControl **pSC,
 		       RoomProcList *room_to_spool,     
 		       HashList *working_ignetcfg,
 		       HashList *the_netmap)
 {
+	struct CitContext *CCC = CC;
 	SpoolControl *sc;
 	int i = 0;
 
@@ -189,16 +205,18 @@ void InspectQueuedRoom(SpoolControl **pSC,
 	}
 	if (sc->room.QRhighest <= sc->lastsent)
 	{
-		syslog(LOG_DEBUG, "nothing to do for <%s>\n", room_to_spool->name);
+		QN_syslog(LOG_DEBUG, "nothing to do for <%s>\n", room_to_spool->name);
 		free(sc);
 		return;
 	}
 
+	begin_critical_section(S_NETCONFIGS);
 	if (sc->RNCfg == NULL)
 		sc->RNCfg = CtdlGetNetCfgForRoom(sc->room.QRnumber);
 
 	if (!HaveSpoolConfig(sc->RNCfg))
 	{
+		end_critical_section(S_NETCONFIGS);
 		free(sc);
 		/* nothing to do for this room... */
 		return;
@@ -216,6 +234,7 @@ void InspectQueuedRoom(SpoolControl **pSC,
 	
 	if (StrLength(sc->RNCfg->Sender) > 0)
 		sc->Users[roommailalias] = NewStrBufDup(sc->RNCfg->Sender);
+	end_critical_section(S_NETCONFIGS);
 
 	sc->next = *pSC;
 	*pSC = sc;
@@ -314,6 +333,7 @@ void CalcListID(SpoolControl *sc)
 	FreeStrBuf(&RoomName);
 }
 
+static time_t last_digest_delivery = 0;
 
 /*
  * Batch up and send all outbound traffic from the current room
@@ -334,22 +354,15 @@ void network_spoolout_room(SpoolControl *sc)
 
 	syslog(LOG_INFO, "Networking started for <%s>\n", CCC->room.QRname);
 
-	/* If there are digest recipients, we have to build a digest */
-	if (sc->Users[digestrecp] != NULL) {
-		sc->digestfp = tmpfile();
-		fprintf(sc->digestfp, "Content-type: text/plain\n\n");
-	}
-
 	CalcListID(sc);
 
 	/* remember where we started... */
 	lastsent = sc->lastsent;
 
-	/* Do something useful */
+	/* Fetch the messages we ought to send & prepare them. */
 	CtdlForEachMessage(MSGS_GT, sc->lastsent, NULL, NULL, NULL,
 		network_spool_msg, sc);
 
-	/* If we wrote a digest, deliver it and then close it */
 	if (StrLength(sc->Users[roommailalias]) > 0)
 	{
 		long len;
@@ -369,28 +382,56 @@ void network_spoolout_room(SpoolControl *sc)
 		buf[i] = tolower(buf[i]);
 		if (isspace(buf[i])) buf[i] = '_';
 	}
+
+
+	/* If we wrote a digest, deliver it and then close it */
+	if (sc->Users[digestrecp] != NULL) {
+		time_t now = time(NULL);
+		time_t secs_today = now % (24 * 60 * 60);
+		long delta = 0;
+
+		if (last_digest_delivery != 0) {
+			delta = now - last_digest_delivery;
+			delta = (24 * 60 * 60) - delta;
+		}
+
+		if ((secs_today < 300) && 
+		    (delta < 300))
+		{
+			if (sc->digestfp == NULL) {
+				sc->digestfp = create_digest_file(&sc->room, 0);
+			}
+			if (sc->digestfp != NULL) {
+				last_digest_delivery = now;
+				fprintf(sc->digestfp,
+					" -----------------------------------"
+					"------------------------------------"
+					"-------\n"
+					"You are subscribed to the '%s' "
+					"list.\n"
+					"To post to the list: %s\n",
+					CCC->room.QRname, buf
+					);
+				network_deliver_digest(sc);	/* deliver */
+				remove_digest_file(&sc->room);
+			}
+		}
+	}
 	if (sc->digestfp != NULL) {
-		fprintf(sc->digestfp,
-			" -----------------------------------"
-			"------------------------------------"
-			"-------\n"
-			"You are subscribed to the '%s' "
-			"list.\n"
-			"To post to the list: %s\n",
-			CCC->room.QRname, buf
-		);
-		network_deliver_digest(sc);	/* deliver and close */
+		fclose(sc->digestfp);
+		sc->digestfp = NULL;
 	}
 
 	/* Now rewrite the config file */
 	if (sc->lastsent != lastsent)
 	{
+		begin_critical_section(S_NETCONFIGS);
 		sc->RNCfg = CtdlGetNetCfgForRoom(sc->room.QRnumber);
 
 		sc->RNCfg->lastsent = sc->lastsent;
 		sc->RNCfg->changed = 1;
+		end_critical_section(S_NETCONFIGS);
 	}
-	end_critical_section(S_NETCONFIGS);
 }
 
 /*
@@ -405,10 +446,9 @@ void network_process_buffer(char *buffer, long size, HashList *working_ignetcfg,
 	struct CtdlMessage *msg = NULL;
 	long pos;
 	int field;
-	struct recptypes *recp = NULL;
+	recptypes *recp = NULL;
 	char target_room[ROOMNAMELEN];
 	struct ser_ret sermsg;
-	char *oldpath = NULL;
 	char filename[PATH_MAX];
 	FILE *fp;
 	const StrBuf *nexthop = NULL;
@@ -439,39 +479,26 @@ void network_process_buffer(char *buffer, long size, HashList *working_ignetcfg,
 	for (pos = 3; pos < size; ++pos) {
 		field = buffer[pos];
 		len = strlen(buffer + pos + 1);
-		msg->cm_fields[field] = malloc(len + 1);
-		memcpy (msg->cm_fields[field], buffer+ pos + 1, len + 1);
+		CM_SetField(msg, field, buffer + pos + 1, len);
 		pos = pos + len + 1;
 	}
 
 	/* Check for message routing */
-	if (msg->cm_fields['D'] != NULL) {
-		if (strcasecmp(msg->cm_fields['D'], config.c_nodename)) {
+	if (!CM_IsEmpty(msg, eDestination)) {
+		if (strcasecmp(msg->cm_fields[eDestination], config.c_nodename)) {
 
 			/* route the message */
-			Buf = NewStrBufPlain(msg->cm_fields['D'], -1);
+			Buf = NewStrBufPlain(CM_KEY(msg,eDestination));
 			if (CtdlIsValidNode(&nexthop, 
 					    NULL, 
 					    Buf, 
 					    working_ignetcfg, 
 					    the_netmap) == 0) 
 			{
-				/* prepend our node to the path */
-				if (msg->cm_fields['P'] != NULL) {
-					oldpath = msg->cm_fields['P'];
-					msg->cm_fields['P'] = NULL;
-				}
-				else {
-					oldpath = strdup("unknown_user");
-				}
-				size = strlen(oldpath) + SIZ;
-				msg->cm_fields['P'] = malloc(size);
-				snprintf(msg->cm_fields['P'], size, "%s!%s",
-					config.c_nodename, oldpath);
-				free(oldpath);
+				Netmap_AddMe(msg, HKEY("unknown_user"));
 
 				/* serialize the message */
-				serialize_message(&sermsg, msg);
+				CtdlSerializeMessage(&sermsg, msg);
 
 				/* now send it */
 				if (StrLength(nexthop) == 0) {
@@ -495,7 +522,7 @@ void network_process_buffer(char *buffer, long size, HashList *working_ignetcfg,
 					QN_syslog(LOG_ERR, "%s: %s\n", filename, strerror(errno));
 				}
 				free(sermsg.ser);
-				CtdlFreeMessage(msg);
+				CM_Free(msg);
 				FreeStrBuf(&Buf);
 				return;
 			}
@@ -520,14 +547,14 @@ void network_process_buffer(char *buffer, long size, HashList *working_ignetcfg,
 	 * connected that it's inevitable.)
 	 */
 	if (network_usetable(msg) != 0) {
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 		return;
 	}
 
 	/* Learn network topology from the path */
-	if ((msg->cm_fields['N'] != NULL) && (msg->cm_fields['P'] != NULL)) {
-		NetworkLearnTopology(msg->cm_fields['N'], 
-				     msg->cm_fields['P'], 
+	if (!CM_IsEmpty(msg, eNodeName) && !CM_IsEmpty(msg, eMessagePath)) {
+		NetworkLearnTopology(msg->cm_fields[eNodeName], 
+				     msg->cm_fields[eMessagePath], 
 				     the_netmap, 
 				     netmap_changed);
 	}
@@ -535,13 +562,13 @@ void network_process_buffer(char *buffer, long size, HashList *working_ignetcfg,
 	/* Is the sending node giving us a very persuasive suggestion about
 	 * which room this message should be saved in?  If so, go with that.
 	 */
-	if (msg->cm_fields['C'] != NULL) {
-		safestrncpy(target_room, msg->cm_fields['C'], sizeof target_room);
+	if (!CM_IsEmpty(msg, eRemoteRoom)) {
+		safestrncpy(target_room, msg->cm_fields[eRemoteRoom], sizeof target_room);
 	}
 
 	/* Otherwise, does it have a recipient?  If so, validate it... */
-	else if (msg->cm_fields['R'] != NULL) {
-		recp = validate_recipients(msg->cm_fields['R'], NULL, 0);
+	else if (!CM_IsEmpty(msg, eRecipient)) {
+		recp = validate_recipients(msg->cm_fields[eRecipient], NULL, 0);
 		if (recp != NULL) if (recp->num_error != 0) {
 			network_bounce(msg,
 				"A message you sent could not be delivered due to an invalid address.\n"
@@ -555,28 +582,22 @@ void network_process_buffer(char *buffer, long size, HashList *working_ignetcfg,
 	}
 
 	/* Our last shot at finding a home for this message is to see if
-	 * it has the O field (Originating room) set.
+	 * it has the eOriginalRoom (O) field (Originating room) set.
 	 */
-	else if (msg->cm_fields['O'] != NULL) {
-		safestrncpy(target_room, msg->cm_fields['O'], sizeof target_room);
+	else if (!CM_IsEmpty(msg, eOriginalRoom)) {
+		safestrncpy(target_room, msg->cm_fields[eOriginalRoom], sizeof target_room);
 	}
 
 	/* Strip out fields that are only relevant during transit */
-	if (msg->cm_fields['D'] != NULL) {
-		free(msg->cm_fields['D']);
-		msg->cm_fields['D'] = NULL;
-	}
-	if (msg->cm_fields['C'] != NULL) {
-		free(msg->cm_fields['C']);
-		msg->cm_fields['C'] = NULL;
-	}
+	CM_FlushField(msg, eDestination);
+	CM_FlushField(msg, eRemoteRoom);
 
 	/* save the message into a room */
 	if (PerformNetprocHooks(msg, target_room) == 0) {
 		msg->cm_flags = CM_SKIP_HOOKS;
 		CtdlSubmitMsg(msg, recp, target_room, 0);
 	}
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 	free_recipients(recp);
 }
 
