@@ -18,6 +18,7 @@
  * 
  */
 
+#include "internet_addressing.h"
 
 #define RSS_UNSET       (1<<0)
 #define RSS_RSS         (1<<1)
@@ -101,6 +102,7 @@ struct rss_aggregator {
 	time_t		 last_error_when;
 	time_t		 next_poll;
 	StrBuf		*Url;
+	StrBuf          *RedirectUrl;
 	StrBuf		*rooms;
 	pRSSConfig       Cfg;
 	HashList	*OtherQRnumbers;
@@ -109,7 +111,7 @@ struct rss_aggregator {
 	StrBuf		*Key;
 
 	rss_item   	*Item;
-	struct recptypes recp;
+	recptypes	 recp;
 	HashPos         *Pos;
 	HashList        *Messages;
 	networker_save_message *ThisMsg;
