@@ -223,12 +223,12 @@ eNextState FinalizeMessageSend_DB(AsyncIO *IO)
 		msg->cm_magic = CTDLMESSAGE_MAGIC;
 		msg->cm_anon_type = MES_NORMAL;
 		msg->cm_format_type = FMT_RFC822;
-		msg->cm_fields['M'] = SmashStrBuf(&Msg->QMsgData);
-		msg->cm_fields['U'] = strdup("QMSG");
+		CM_SetAsFieldSB(msg, eMesageText, &Msg->QMsgData);
+		CM_SetField(msg, eMsgSubject, HKEY("QMSG"));
 		Msg->MyQItem->QueMsgID =
 			CtdlSubmitMsg(msg, NULL, SMTP_SPOOLOUT_ROOM, QP_EADDR);
 		EVS_syslog(LOG_DEBUG, "%ld", Msg->MyQItem->QueMsgID);
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 	}
 	else {
 		CtdlDeleteMessages(SMTP_SPOOLOUT_ROOM,
@@ -255,7 +255,7 @@ eNextState Terminate(AsyncIO *IO)
 eNextState FinalizeMessageSend(SmtpOutMsg *Msg)
 {
 	/* hand over to DB Queue */
-	return EventQueueDBOperation(&Msg->IO, FinalizeMessageSend_DB);
+	return EventQueueDBOperation(&Msg->IO, FinalizeMessageSend_DB, 0);
 }
 
 eNextState FailOneAttempt(AsyncIO *IO)
@@ -288,12 +288,12 @@ eNextState FailOneAttempt(AsyncIO *IO)
 	    Msg->MyQItem->HaveRelay)
 	{
 		EVS_syslog(LOG_DEBUG, "%s Aborting; last relay failed.\n", __FUNCTION__);
-		return eAbort;
+		return FinalizeMessageSend(Msg);
 	}
 
 	if (Msg->pCurrRelay == NULL) {
 		EVS_syslog(LOG_DEBUG, "%s Aborting\n", __FUNCTION__);
-		return eAbort;
+		return FinalizeMessageSend(Msg);
 	}
 	if (Msg->pCurrRelay->IsIP) {
 		EVS_syslog(LOG_DEBUG, "%s connecting IP\n", __FUNCTION__);
@@ -357,7 +357,7 @@ eNextState mx_connect_ip(AsyncIO *IO)
 	SmtpOutMsg *Msg = IO->Data;
 	SetSMTPState(IO, eSTMPconnecting);
 
-	EVS_syslog(LOG_DEBUG, "%s\n", __FUNCTION__);
+	EVS_syslog(LOG_DEBUG, "%s(%s)\n", __FUNCTION__, (Msg->IsRelay)? "Relay":"Remote");
 
 	IO->ConnectMe = Msg->pCurrRelay;
 	Msg->State = eConnectMX;
@@ -657,6 +657,7 @@ void smtp_try_one_queue_entry(OneQueItem *MyQItem,
 					  resolve_mx_records);
 		}
 		else { /* oh... via relay host */
+			Msg->IsRelay = 1;
 			if (Msg->pCurrRelay->IsIP) {
 				SetSMTPState(&Msg->IO, eSTMPconnecting);
 				QueueEventContext(&Msg->IO,
