@@ -45,6 +45,7 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
+#include "room_ops.h"
 #include "database.h"
 #include "msgbase.h"
 #include "euidindex.h"
@@ -69,7 +70,7 @@ char *wwm = "9999999999.WikiWaybackMachine";
  * Before allowing a wiki page save to execute, we have to perform version control.
  * This involves fetching the old version of the page if it exists.
  */
-int wiki_upload_beforesave(struct CtdlMessage *msg) {
+int wiki_upload_beforesave(struct CtdlMessage *msg, recptypes *recp) {
 	struct CitContext *CCC = CC;
 	long old_msgnum = (-1L);
 	struct CtdlMessage *old_msg = NULL;
@@ -82,6 +83,7 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 	FILE *fp;
 	int rv;
 	char history_page[1024];
+	long history_page_len;
 	char boundary[256];
 	char prefixed_boundary[258];
 	char buf[1024];
@@ -92,7 +94,8 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 	if (!CCC->logged_in) return(0);	/* Only do this if logged in. */
 
 	/* Is this a room with a Wiki in it, don't run this hook. */
-	if (CCC->room.QRdefaultview != VIEW_WIKI) {
+	if ((CCC->room.QRdefaultview != VIEW_WIKI) &&
+	    (CCC->room.QRdefaultview != VIEW_WIKIMD)) {
 		return(0);
 	}
 
@@ -100,31 +103,29 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 	if (msg->cm_format_type != 4) return(0);
 
 	/* If there's no EUID we can't do this.  Reject the post. */
-	if (msg->cm_fields['E'] == NULL) return(1);
+	if (CM_IsEmpty(msg, eExclusiveID)) return(1);
 
-	snprintf(history_page, sizeof history_page, "%s_HISTORY_", msg->cm_fields['E']);
+	history_page_len = snprintf(history_page, sizeof history_page,
+				    "%s_HISTORY_", msg->cm_fields[eExclusiveID]);
 
 	/* Make sure we're saving a real wiki page rather than a wiki history page.
 	 * This is important in order to avoid recursing infinitely into this hook.
 	 */
-	if (	(strlen(msg->cm_fields['E']) >= 9)
-		&& (!strcasecmp(&msg->cm_fields['E'][strlen(msg->cm_fields['E'])-9], "_HISTORY_"))
+	if (	(msg->cm_lengths[eExclusiveID] >= 9)
+		&& (!strcasecmp(&msg->cm_fields[eExclusiveID][msg->cm_lengths[eExclusiveID]-9], "_HISTORY_"))
 	) {
 		syslog(LOG_DEBUG, "History page not being historied\n");
 		return(0);
 	}
 
 	/* If there's no message text, obviously this is all b0rken and shouldn't happen at all */
-	if (msg->cm_fields['M'] == NULL) return(0);
+	if (CM_IsEmpty(msg, eMesageText)) return(0);
 
 	/* Set the message subject identical to the page name */
-	if (msg->cm_fields['U'] != NULL) {
-		free(msg->cm_fields['U']);
-	}
-	msg->cm_fields['U'] = strdup(msg->cm_fields['E']);
+	CM_CopyField(msg, eMsgSubject, eExclusiveID);
 
 	/* See if we can retrieve the previous version. */
-	old_msgnum = CtdlLocateMessageByEuid(msg->cm_fields['E'], &CCC->room);
+	old_msgnum = CtdlLocateMessageByEuid(msg->cm_fields[eExclusiveID], &CCC->room);
 	if (old_msgnum > 0L) {
 		old_msg = CtdlFetchMessage(old_msgnum, 1);
 	}
@@ -132,14 +133,14 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 		old_msg = NULL;
 	}
 
-	if ((old_msg != NULL) && (old_msg->cm_fields['M'] == NULL)) {	/* old version is corrupt? */
-		CtdlFreeMessage(old_msg);
+	if ((old_msg != NULL) && (CM_IsEmpty(old_msg, eMesageText))) {	/* old version is corrupt? */
+		CM_Free(old_msg);
 		old_msg = NULL;
 	}
 	
 	/* If no changes were made, don't bother saving it again */
-	if ((old_msg != NULL) && (!strcmp(msg->cm_fields['M'], old_msg->cm_fields['M']))) {
-		CtdlFreeMessage(old_msg);
+	if ((old_msg != NULL) && (!strcmp(msg->cm_fields[eMesageText], old_msg->cm_fields[eMesageText]))) {
+		CM_Free(old_msg);
 		return(1);
 	}
 
@@ -152,13 +153,13 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 
 	if (old_msg != NULL) {
 		fp = fopen(diff_old_filename, "w");
-		rv = fwrite(old_msg->cm_fields['M'], strlen(old_msg->cm_fields['M']), 1, fp);
+		rv = fwrite(old_msg->cm_fields[eMesageText], old_msg->cm_lengths[eMesageText], 1, fp);
 		fclose(fp);
-		CtdlFreeMessage(old_msg);
+		CM_Free(old_msg);
 	}
 
 	fp = fopen(diff_new_filename, "w");
-	rv = fwrite(msg->cm_fields['M'], strlen(msg->cm_fields['M']), 1, fp);
+	rv = fwrite(msg->cm_fields[eMesageText], msg->cm_lengths[eMesageText], 1, fp);
 	fclose(fp);
 
 	snprintf(diff_cmd, sizeof diff_cmd,
@@ -183,7 +184,7 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 		fseek(fp, 0L, SEEK_SET);
 		diffbuf = malloc(diffbuf_len + 1);
 		fread(diffbuf, diffbuf_len, 1, fp);
-		diffbuf[diffbuf_len] = 0;
+		diffbuf[diffbuf_len] = '\0';
 		fclose(fp);
 	}
 
@@ -212,27 +213,30 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 
 	/* Create a new history message if necessary */
 	if (history_msg == NULL) {
+		char *buf;
+		long len;
+
 		history_msg = malloc(sizeof(struct CtdlMessage));
 		memset(history_msg, 0, sizeof(struct CtdlMessage));
 		history_msg->cm_magic = CTDLMESSAGE_MAGIC;
 		history_msg->cm_anon_type = MES_NORMAL;
 		history_msg->cm_format_type = FMT_RFC822;
-		history_msg->cm_fields['A'] = strdup("Citadel");
-		history_msg->cm_fields['R'] = strdup(CCC->room.QRname);
-		history_msg->cm_fields['E'] = strdup(history_page);
-		history_msg->cm_fields['U'] = strdup(history_page);
-		history_msg->cm_fields['1'] = strdup("1");		/* suppress full text indexing */
+		CM_SetField(history_msg, eAuthor, HKEY("Citadel"));
+		CM_SetField(history_msg, eRecipient, CCC->room.QRname, strlen(CCC->room.QRname));
+		CM_SetField(history_msg, eExclusiveID, history_page, history_page_len);
+		CM_SetField(history_msg, eMsgSubject, history_page, history_page_len);
+		CM_SetField(history_msg, eSuppressIdx, HKEY("1")); /* suppress full text indexing */
 		snprintf(boundary, sizeof boundary, "Citadel--Multipart--%04x--%08lx", getpid(), time(NULL));
-		history_msg->cm_fields['M'] = malloc(1024);
-		snprintf(history_msg->cm_fields['M'], 1024,
-			"Content-type: multipart/mixed; boundary=\"%s\"\n\n"
-			"This is a Citadel wiki history encoded as multipart MIME.\n"
-			"Each part is comprised of a diff script representing one change set.\n"
-			"\n"
-			"--%s--\n"
-			,
-			boundary, boundary
+		buf = (char*) malloc(1024);
+		len = snprintf(buf, 1024,
+			       "Content-type: multipart/mixed; boundary=\"%s\"\n\n"
+			       "This is a Citadel wiki history encoded as multipart MIME.\n"
+			       "Each part is comprised of a diff script representing one change set.\n"
+			       "\n"
+			       "--%s--\n",
+			       boundary, boundary
 		);
+		CM_SetAsField(history_msg, eMesageText, &buf, len);
 	}
 
 	/* Update the history message (regardless of whether it's new or existing) */
@@ -240,17 +244,14 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 	/* Remove the Message-ID from the old version of the history message.  This will cause a brand
 	 * new one to be generated, avoiding an uninitentional hit of the loop zapper when we replicate.
 	 */
-	if (history_msg->cm_fields['I'] != NULL) {
-		free(history_msg->cm_fields['I']);
-		history_msg->cm_fields['I'] = NULL;
-	}
+	CM_FlushField(history_msg, emessageId);
 
 	/* Figure out the boundary string.  We do this even when we generated the
 	 * boundary string in the above code, just to be safe and consistent.
 	 */
-	strcpy(boundary, "");
+	*boundary = '\0';
 
-	ptr = history_msg->cm_fields['M'];
+	ptr = history_msg->cm_fields[eMesageText];
 	do {
 		ptr = memreadline(ptr, buf, sizeof buf);
 		if (*ptr != 0) {
@@ -279,44 +280,62 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 	 * Now look for the first boundary.  That is where we need to insert our fun.
 	 */
 	if (!IsEmptyStr(boundary)) {
-		snprintf(prefixed_boundary, sizeof prefixed_boundary, "--%s", boundary);
-		history_msg->cm_fields['M'] = realloc(history_msg->cm_fields['M'],
-			strlen(history_msg->cm_fields['M']) + strlen(diffbuf) + 1024
-		);
-		ptr = bmstrcasestr(history_msg->cm_fields['M'], prefixed_boundary);
+		char *MsgText;
+		long MsgTextLen;
+		time_t Now = time(NULL);
+
+		snprintf(prefixed_boundary, sizeof(prefixed_boundary), "--%s", boundary);
+		
+		CM_GetAsField(history_msg, eMesageText, &MsgText, &MsgTextLen);
+
+		ptr = bmstrcasestr(MsgText, prefixed_boundary);
 		if (ptr != NULL) {
-			char *the_rest_of_it = strdup(ptr);
+			StrBuf *NewMsgText;
 			char uuid[64];
 			char memo[512];
+			long memolen;
 			char encoded_memo[1024];
+			
+			NewMsgText = NewStrBufPlain(NULL, MsgTextLen + diffbuf_len + 1024);
+
 			generate_uuid(uuid);
-			snprintf(memo, sizeof memo, "%s|%ld|%s|%s", 
-				uuid,
-				time(NULL),
-				CCC->user.fullname,
-				config.c_nodename
-			);
-			CtdlEncodeBase64(encoded_memo, memo, strlen(memo), 0);
-			sprintf(ptr, "--%s\n"
-					"Content-type: text/plain\n"
-					"Content-Disposition: inline; filename=\"%s\"\n"
-					"Content-Transfer-Encoding: 8bit\n"
-					"\n"
-					"%s\n"
-					"%s"
-					,
-				boundary,
-				encoded_memo,
-				diffbuf,
-				the_rest_of_it
-			);
-			free(the_rest_of_it);
+			memolen = snprintf(memo, sizeof(memo), "%s|%ld|%s|%s", 
+					   uuid,
+					   Now,
+					   CCC->user.fullname,
+					   config.c_nodename);
+
+			memolen = CtdlEncodeBase64(encoded_memo, memo, memolen, 0);
+
+			StrBufAppendBufPlain(NewMsgText, HKEY("--"), 0);
+			StrBufAppendBufPlain(NewMsgText, boundary, -1, 0);
+			StrBufAppendBufPlain(
+				NewMsgText, 
+				HKEY("\n"
+				     "Content-type: text/plain\n"
+				     "Content-Disposition: inline; filename=\""), 0);
+
+			StrBufAppendBufPlain(NewMsgText, encoded_memo, memolen, 0);
+
+			StrBufAppendBufPlain(
+				NewMsgText, 
+				HKEY("\"\n"
+				     "Content-Transfer-Encoding: 8bit\n"
+				     "\n"), 0);
+
+			StrBufAppendBufPlain(NewMsgText, diffbuf, diffbuf_len, 0);
+			StrBufAppendBufPlain(NewMsgText, HKEY("\n"), 0);
+
+			StrBufAppendBufPlain(NewMsgText, ptr, MsgTextLen - (ptr - MsgText), 0);
+			free(MsgText);
+			CM_SetAsFieldSB(history_msg, eMesageText, &NewMsgText); 
+		}
+		else
+		{
+			CM_SetAsField(history_msg, eMesageText, &MsgText, MsgTextLen); 
 		}
 
-		history_msg->cm_fields['T'] = realloc(history_msg->cm_fields['T'], 32);
-		if (history_msg->cm_fields['T'] != NULL) {
-			snprintf(history_msg->cm_fields['T'], 32, "%ld", time(NULL));
-		}
+		CM_SetFieldLONG(history_msg, eTimestamp, Now);
 	
 		CtdlSubmitMsg(history_msg, NULL, "", 0);
 	}
@@ -325,7 +344,7 @@ int wiki_upload_beforesave(struct CtdlMessage *msg) {
 	}
 
 	free(diffbuf);
-	free(history_msg);
+	CM_Free(history_msg);
 	return(0);
 }
 
@@ -377,8 +396,8 @@ void wiki_history(char *pagename) {
 		msg = NULL;
 	}
 
-	if ((msg != NULL) && (msg->cm_fields['M'] == NULL)) {
-		CtdlFreeMessage(msg);
+	if ((msg != NULL) && CM_IsEmpty(msg, eMesageText)) {
+		CM_Free(msg);
 		msg = NULL;
 	}
 
@@ -389,10 +408,10 @@ void wiki_history(char *pagename) {
 
 	
 	cprintf("%d Revision history for '%s'\n", LISTING_FOLLOWS, pagename);
-	mime_parser(msg->cm_fields['M'], NULL, *wiki_history_callback, NULL, NULL, NULL, 0);
+	mime_parser(CM_RANGE(msg, eMesageText), *wiki_history_callback, NULL, NULL, NULL, 0);
 	cprintf("000\n");
 
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 	return;
 }
 
@@ -468,11 +487,11 @@ void wiki_rev_callback(char *name, char *filename, char *partnum, char *disp,
  */
 void wiki_rev(char *pagename, char *rev, char *operation)
 {
+	struct CitContext *CCC = CC;
 	int r;
 	char history_page_name[270];
 	long msgnum;
 	char temp[PATH_MAX];
-	char timestamp[64];
 	struct CtdlMessage *msg;
 	FILE *fp;
 	struct HistoryEraserCallBackData hecbd;
@@ -501,7 +520,7 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 	/* Begin by fetching the current version of the page.  We're going to patch
 	 * backwards through the diffs until we get the one we want.
 	 */
-	msgnum = CtdlLocateMessageByEuid(pagename, &CC->room);
+	msgnum = CtdlLocateMessageByEuid(pagename, &CCC->room);
 	if (msgnum > 0L) {
 		msg = CtdlFetchMessage(msgnum, 1);
 	}
@@ -509,8 +528,8 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 		msg = NULL;
 	}
 
-	if ((msg != NULL) && (msg->cm_fields['M'] == NULL)) {
-		CtdlFreeMessage(msg);
+	if ((msg != NULL) && CM_IsEmpty(msg, eMesageText)) {
+		CM_Free(msg);
 		msg = NULL;
 	}
 
@@ -524,18 +543,18 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 	CtdlMakeTempFileName(temp, sizeof temp);
 	fp = fopen(temp, "w");
 	if (fp != NULL) {
-		r = fwrite(msg->cm_fields['M'], strlen(msg->cm_fields['M']), 1, fp);
+		r = fwrite(msg->cm_fields[eMesageText], msg->cm_lengths[eMesageText], 1, fp);
 		fclose(fp);
 	}
 	else {
 		syslog(LOG_ALERT, "Cannot open %s: %s\n", temp, strerror(errno));
 	}
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 
 	/* Get the revision history */
 
 	snprintf(history_page_name, sizeof history_page_name, "%s_HISTORY_", pagename);
-	msgnum = CtdlLocateMessageByEuid(history_page_name, &CC->room);
+	msgnum = CtdlLocateMessageByEuid(history_page_name, &CCC->room);
 	if (msgnum > 0L) {
 		msg = CtdlFetchMessage(msgnum, 1);
 	}
@@ -543,8 +562,8 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 		msg = NULL;
 	}
 
-	if ((msg != NULL) && (msg->cm_fields['M'] == NULL)) {
-		CtdlFreeMessage(msg);
+	if ((msg != NULL) && CM_IsEmpty(msg, eMesageText)) {
+		CM_Free(msg);
 		msg = NULL;
 	}
 
@@ -562,8 +581,8 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 	hecbd.stop_when = rev;
 	striplt(hecbd.stop_when);
 
-	mime_parser(msg->cm_fields['M'], NULL, *wiki_rev_callback, NULL, NULL, (void *)&hecbd, 0);
-	CtdlFreeMessage(msg);
+	mime_parser(CM_RANGE(msg, eMesageText), *wiki_rev_callback, NULL, NULL, (void *)&hecbd, 0);
+	CM_Free(msg);
 
 	/* Were we successful? */
 	if (hecbd.done == 0) {
@@ -582,20 +601,22 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 		msg->cm_format_type = FMT_RFC822;
 		fp = fopen(temp, "r");
 		if (fp) {
+			char *msgbuf;
 			fseek(fp, 0L, SEEK_END);
 			len = ftell(fp);
 			fseek(fp, 0L, SEEK_SET);
-			msg->cm_fields['M'] = malloc(len + 1);
-			rv = fread(msg->cm_fields['M'], len, 1, fp);
+			msgbuf = malloc(len + 1);
+			rv = fread(msgbuf, len, 1, fp);
 			syslog(LOG_DEBUG, "did %d blocks of %ld bytes\n", rv, len);
-			msg->cm_fields['M'][len] = 0;
+			msgbuf[len] = '\0';
+			CM_SetAsField(msg, eMesageText, &msgbuf, len);
 			fclose(fp);
 		}
 		if (len <= 0) {
 			msgnum = (-1L);
 		}
 		else if (!strcasecmp(operation, "fetch")) {
-			msg->cm_fields['A'] = strdup("Citadel");
+			CM_SetField(msg, eAuthor, HKEY("Citadel"));
 			CtdlCreateRoom(wwm, 5, "", 0, 1, 1, VIEW_BBS);	/* Not an error if already exists */
 			msgnum = CtdlSubmitMsg(msg, NULL, wwm, 0);	/* Store the revision here */
 
@@ -605,33 +626,32 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 			 * but only if the client fetches the message we just generated immediately
 			 * without first trying to perform other fetch operations.
 			 */
-			if (CC->cached_msglist != NULL) {
-				free(CC->cached_msglist);
-				CC->cached_msglist = NULL;
-				CC->cached_num_msgs = 0;
+			if (CCC->cached_msglist != NULL) {
+				free(CCC->cached_msglist);
+				CCC->cached_msglist = NULL;
+				CCC->cached_num_msgs = 0;
 			}
-			CC->cached_msglist = malloc(sizeof(long));
-			if (CC->cached_msglist != NULL) {
-				CC->cached_num_msgs = 1;
-				CC->cached_msglist[0] = msgnum;
+			CCC->cached_msglist = malloc(sizeof(long));
+			if (CCC->cached_msglist != NULL) {
+				CCC->cached_num_msgs = 1;
+				CCC->cached_msglist[0] = msgnum;
 			}
 
 		}
 		else if (!strcasecmp(operation, "revert")) {
-			snprintf(timestamp, sizeof timestamp, "%ld", time(NULL));
-			msg->cm_fields['T'] = strdup(timestamp);
-			msg->cm_fields['A'] = strdup(CC->user.fullname);
-			msg->cm_fields['F'] = strdup(CC->cs_inet_email);
-			msg->cm_fields['O'] = strdup(CC->room.QRname);
-			msg->cm_fields['N'] = strdup(NODENAME);
-			msg->cm_fields['E'] = strdup(pagename);
+			CM_SetFieldLONG(msg, eTimestamp, time(NULL));
+			CM_SetField(msg, eAuthor, CCC->user.fullname, strlen(CCC->user.fullname));
+			CM_SetField(msg, erFc822Addr, CCC->cs_inet_email, strlen(CCC->cs_inet_email));
+			CM_SetField(msg, eOriginalRoom, CCC->room.QRname, strlen(CCC->room.QRname));
+			CM_SetField(msg, eNodeName, CFG_KEY(c_nodename));
+			CM_SetField(msg, eExclusiveID, pagename, strlen(pagename));
 			msgnum = CtdlSubmitMsg(msg, NULL, "", 0);	/* Replace the current revision */
 		}
 		else {
 			/* Theoretically it is impossible to get here, but throw an error anyway */
 			msgnum = (-1L);
 		}
-		CtdlFreeMessage(msg);
+		CM_Free(msg);
 		if (msgnum >= 0L) {
 			cprintf("%d %ld\n", CIT_OK, msgnum);		/* Give the client a msgnum */
 		}
