@@ -1,7 +1,7 @@
 /*
  * Handle XMPP presence exchanges
  *
- * Copyright (c) 2007-2010 by Art Cancro
+ * Copyright (c) 2007-2015 by Art Cancro and citadel.org
  *
  * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,7 @@ void xmpp_indicate_presence(char *presence_jid)
 {
 	char xmlbuf[256];
 
+	XMPP_syslog(LOG_DEBUG, "XMPP: indicating presence of <%s> to <%s>", presence_jid, XMPP->client_jid);
 	cprintf("<presence from=\"%s\" ", xmlesc(xmlbuf, presence_jid, sizeof xmlbuf));
 	cprintf("to=\"%s\"></presence>", xmlesc(xmlbuf, XMPP->client_jid, sizeof xmlbuf));
 }
@@ -147,7 +148,7 @@ void xmpp_destroy_buddy(char *presence_jid, int aggressively) {
 		);
 	}
 
-	// FIXME ... we should implement xmpp_indicate_nonpresence so we can use it elsewhere
+	// note: we should implement xmpp_indicate_nonpresence so we can use it elsewhere
 
 	/* Do an unsolicited roster update that deletes the contact. */
 	cprintf("<iq from=\"%s\" to=\"%s\" id=\"unbuddy_%x\" type=\"result\">",
@@ -171,7 +172,7 @@ void xmpp_destroy_buddy(char *presence_jid, int aggressively) {
  */
 void xmpp_presence_notify(char *presence_jid, int event_type) {
 	struct CitContext *cptr;
-	static int unsolicited_id;
+	static int unsolicited_id = 12345;
 	int visible_sessions = 0;
 	int nContexts, i;
 	int which_cptr_is_relevant = (-1);
@@ -194,13 +195,11 @@ void xmpp_presence_notify(char *presence_jid, int event_type) {
 		}
 	}
 
-	XMPP_syslog(LOG_DEBUG, "%d sessions for <%s> are now visible to session %d\n",
-		    visible_sessions, presence_jid, CC->cs_pid);
+	syslog(LOG_DEBUG, "%d sessions for <%s> are now visible to session %d\n", visible_sessions, presence_jid, CC->cs_pid);
 
 	if ( (event_type == XMPP_EVT_LOGIN) && (visible_sessions == 1) ) {
 
-		XMPP_syslog(LOG_DEBUG, "Telling session %d that <%s> logged in\n",
-			    CC->cs_pid, presence_jid);
+		syslog(LOG_DEBUG, "Telling session %d that <%s> logged in\n", CC->cs_pid, presence_jid);
 
 		/* Do an unsolicited roster update that adds a new contact. */
 		assert(which_cptr_is_relevant >= 0);
@@ -214,7 +213,7 @@ void xmpp_presence_notify(char *presence_jid, int event_type) {
 	}
 
 	if (visible_sessions == 0) {
-		XMPP_syslog(LOG_DEBUG, "Telling session %d that <%s> logged out\n",
+		syslog(LOG_DEBUG, "Telling session %d that <%s> logged out\n",
 			    CC->cs_pid, presence_jid);
 		xmpp_destroy_buddy(presence_jid, 0);	/* non aggressive presence update */
 	}
@@ -238,12 +237,12 @@ void xmpp_fetch_mortuary_backend(long msgnum, void *userdata) {
 	/* now add anyone we find into the hashlist */
 
 	/* skip past the headers */
-	ptr = strstr(msg->cm_fields['M'], "\n\n");
+	ptr = strstr(msg->cm_fields[eMesageText], "\n\n");
 	if (ptr != NULL) {
 		ptr += 2;
 	}
 	else {
-		ptr = strstr(msg->cm_fields['M'], "\n\r\n");
+		ptr = strstr(msg->cm_fields[eMesageText], "\n\r\n");
 		if (ptr != NULL) {
 			ptr += 3;
 		}
@@ -259,7 +258,7 @@ void xmpp_fetch_mortuary_backend(long msgnum, void *userdata) {
 		}
 	}
 
-	CtdlFreeMessage(msg);
+	CM_Free(msg);
 }
 
 
@@ -271,7 +270,7 @@ void xmpp_fetch_mortuary_backend(long msgnum, void *userdata) {
 HashList *xmpp_fetch_mortuary(void) {
 	HashList *mortuary = NewHash(1, NULL);
 	if (!mortuary) {
-		XMPPM_syslog(LOG_ALERT, "NewHash() failed!\n");
+		syslog(LOG_ALERT, "NewHash() failed!\n");
 		return(NULL);
 	}
 
@@ -311,14 +310,6 @@ void xmpp_store_mortuary(HashList *mortuary) {
 	}
 	DeleteHashPos(&HashPos);
 
-	/* FIXME temp crap 
-	StrBufAppendPrintf(themsg, "foo@bar.com\n");
-	StrBufAppendPrintf(themsg, "baz@quux.com\n");
-	StrBufAppendPrintf(themsg, "haha%c\n", 1);
-	StrBufAppendPrintf(themsg, "baaaz@quux.com\n");
-	StrBufAppendPrintf(themsg, "baaaz@quuuuuux.com\n");
-	*/
-
 	/* Delete the old mortuary */
 	CtdlDeleteMessages(USERCONFIGROOM, NULL, 0, XMPPMORTUARY);
 
