@@ -1,21 +1,15 @@
 /*
  * Handle <iq> <get> <query> type situations (namespace queries)
  *
- * Copyright (c) 2007-2009 by Art Cancro
+ * Copyright (c) 2007-2015 by Art Cancro and citadel.org
  *
- *  This program is open source software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 3.
- *  
- *  
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  
- *  
- *  
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  */
 
 #include "sysdep.h"
@@ -71,6 +65,7 @@ void xmpp_roster_item(struct CitContext *cptr) {
 	cprintf("</item>");
 }
 
+
 /* 
  * Return the results for a "jabber:iq:roster:query"
  *
@@ -83,6 +78,7 @@ void xmpp_iq_roster_query(void)
 	struct CitContext *cptr;
 	int nContexts, i;
 
+	syslog(LOG_DEBUG, "Roster push!");
 	cprintf("<query xmlns=\"jabber:iq:roster\">");
 	cptr = CtdlGetContextArray(&nContexts);
 	if (cptr) {
@@ -98,20 +94,17 @@ void xmpp_iq_roster_query(void)
 }
 
 
-/*
- * TODO: handle queries on some or all of these namespaces
- *
-xmpp_query_namespace(purple5b5c1e58, splorph.xand.com, http://jabber.org/protocol/disco#items:query)
-xmpp_query_namespace(purple5b5c1e59, splorph.xand.com, http://jabber.org/protocol/disco#info:query)
-xmpp_query_namespace(purple5b5c1e5a, , vcard-temp:query)
- *
- */
 
+/*
+ * Client is doing a namespace query.  These are all handled differently.
+ * A "rumplestiltskin lookup" is the most efficient way to handle this.  Please do not refactor this code.
+ */
 void xmpp_query_namespace(char *iq_id, char *iq_from, char *iq_to, char *query_xmlns)
 {
 	int supported_namespace = 0;
 	int roster_query = 0;
 	char xmlbuf[256];
+	int reply_must_be_from_my_jid = 0;
 
 	/* We need to know before we begin the response whether this is a supported namespace, so
 	 * unfortunately all supported namespaces need to be defined here *and* down below where
@@ -120,20 +113,50 @@ void xmpp_query_namespace(char *iq_id, char *iq_from, char *iq_to, char *query_x
 	if (
 		(!strcasecmp(query_xmlns, "jabber:iq:roster:query"))
 		|| (!strcasecmp(query_xmlns, "jabber:iq:auth:query"))
+		|| (!strcasecmp(query_xmlns, "http://jabber.org/protocol/disco#items:query"))
+		|| (!strcasecmp(query_xmlns, "http://jabber.org/protocol/disco#info:query"))
 	) {
 		supported_namespace = 1;
 	}
 
-	XMPP_syslog(LOG_DEBUG, "xmpp_query_namespace(%s, %s, %s, %s)\n", iq_id, iq_from, iq_to, query_xmlns);
+	XMPP_syslog(LOG_DEBUG, "xmpp_query_namespace(id=%s, from=%s, to=%s, xmlns=%s)\n", iq_id, iq_from, iq_to, query_xmlns);
 
 	/*
 	 * Beginning of query result.
 	 */
-	if (supported_namespace) {
-		cprintf("<iq type=\"result\" ");
+
+	if (!strcasecmp(query_xmlns, "jabber:iq:roster:query")) {
+		reply_must_be_from_my_jid = 1;
+	}
+
+	char dom[1024];								// client is expecting to see the reply
+	if (reply_must_be_from_my_jid) {					// coming "from" the user's jid
+		safestrncpy(dom, XMPP->client_jid, sizeof(dom));
+		char *slash = strchr(dom, '/');
+		if (slash) {
+			*slash = 0;
+		}
 	}
 	else {
-		cprintf("<iq type=\"error\" ");
+		safestrncpy(dom, XMPP->client_jid, sizeof(dom));		// client is expecting to see the reply
+		if (IsEmptyStr(dom)) {						// coming "from" the domain of the user's jid
+			safestrncpy(dom, XMPP->server_name, sizeof(dom));
+		}
+		char *at = strrchr(dom, '@');
+		if (at) {
+			strcpy(dom, ++at);
+		}
+		char *slash = strchr(dom, '/');
+		if (slash) {
+			*slash = 0;
+		}
+	}
+
+	if (supported_namespace) {
+		cprintf("<iq type=\"result\" from=\"%s\" ", xmlesc(xmlbuf, dom, sizeof xmlbuf) );
+	}
+	else {
+		cprintf("<iq type=\"error\" from=\"%s\" ", xmlesc(xmlbuf, dom, sizeof xmlbuf) );
 	}
 	if (!IsEmptyStr(iq_from)) {
 		cprintf("to=\"%s\" ", xmlesc(xmlbuf, iq_from, sizeof xmlbuf));
@@ -156,16 +179,23 @@ void xmpp_query_namespace(char *iq_id, char *iq_from, char *iq_to, char *query_x
 		);
 	}
 
+	// Extension "xep-0030" (http://xmpp.org/extensions/xep-0030.html) (return an empty set of results)
+	else if (!strcasecmp(query_xmlns, "http://jabber.org/protocol/disco#items:query")) {
+		cprintf("<query xmlns=\"%s\"/>", xmlesc(xmlbuf, query_xmlns, sizeof xmlbuf));
+	}
+
+	// Extension "xep-0030" (http://xmpp.org/extensions/xep-0030.html) (return an empty set of results)
+	else if (!strcasecmp(query_xmlns, "http://jabber.org/protocol/disco#info:query")) {
+		cprintf("<query xmlns=\"%s\"/>", xmlesc(xmlbuf, query_xmlns, sizeof xmlbuf));
+	}
+
 	/*
 	 * If we didn't hit any known query namespaces then we should deliver a
 	 * "service unavailable" error (see RFC3921 section 2.4 and 11.1.5.4)
 	 */
 
 	else {
-		XMPP_syslog(LOG_DEBUG,
-			    "Unknown query namespace '%s' - returning <service-unavailable/>\n",
-			    query_xmlns
-		);
+		XMPP_syslog(LOG_DEBUG, "Unknown query namespace '%s' - returning <service-unavailable/>\n", query_xmlns);
 		cprintf("<error code=\"503\" type=\"cancel\">"
 			"<service-unavailable xmlns=\"urn:ietf:params:xml:ns:xmpp-stanzas\"/>"
 			"</error>"
