@@ -13,26 +13,13 @@
  * GNU General Public License for more details.
  */
 
-#include "sysdep.h"
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <dirent.h>
-#include <string.h>
-#include <limits.h>
-#include <ctype.h>
-#include <syslog.h>
 #include <libcitadel.h>
-#include "citadel.h"
-#include "server.h"
-#include "serv_extensions.h"
-#include "sysdep_decls.h"
-#include "msgbase.h"
-#include "config.h"
 
+#include "sysdep_decls.h"
 #include "modules/crypto/serv_crypto.h"	/* Needed until a universal crypto startup hook is implimented for CtdlStartTLS */
 
+#include "serv_extensions.h"
 #include "ctdl_module.h"
 
 
@@ -42,12 +29,6 @@ int DebugModules = 0;
  * Structure defentitions for hook tables
  */
 
-typedef struct __LogDebugEntry {
-	CtdlDbgFunction F;
-	const char *Name;
-	long Len;
-	const int *LogP;
-} LogDebugEntry;
 HashList *LogDebugEntryTable = NULL;
 
 typedef struct LogFunctionHook LogFunctionHook;
@@ -120,7 +101,7 @@ UserFunctionHook *UserHookTable = NULL;
 typedef struct MessageFunctionHook MessageFunctionHook;
 struct MessageFunctionHook {
 	MessageFunctionHook *next;
-	int (*h_function_pointer) (struct CtdlMessage *msg);
+	int (*h_function_pointer) (struct CtdlMessage *msg, recptypes *recps);
 	int eventtype;
 };
 MessageFunctionHook *MessageHookTable = NULL;
@@ -392,52 +373,6 @@ void CtdlSetDebugLogFacilities(const char **Str, long n)
 		DeleteHashPos(&Pos);
 	}
 }
-void cmd_log_get(char *argbuf)
-{
-	long HKLen;
-	const char *ch;
-	HashPos *Pos;
-	void *vptr;
-
-	if (CtdlAccessCheck(ac_aide)) return;
-
-	cprintf("%d Log modules enabled:\n", LISTING_FOLLOWS);
-
-	Pos = GetNewHashPos(LogDebugEntryTable, 0);
-
-	while (GetNextHashPos(LogDebugEntryTable, Pos, &HKLen, &ch, &vptr)) {
-		LogDebugEntry *E = (LogDebugEntry*)vptr;
-		cprintf("%s|%d\n", ch, *E->LogP);
-	}
-	
-	DeleteHashPos(&Pos);
-	cprintf("000\n");
-}
-void cmd_log_set(char *argbuf)
-{
-	void *vptr;
-	int lset;
-	int wlen;
-	char which[SIZ] = "";
-
-	if (CtdlAccessCheck(ac_aide)) return;
-
-	wlen = extract_token(which, argbuf, 0, '|', sizeof(which));
-	if (wlen < 0) wlen = 0;
-	lset = extract_int(argbuf, 1);
-	if (lset != 0) lset = 1;
-	if (GetHash(LogDebugEntryTable, which, wlen, &vptr) && 
-	    (vptr != NULL))
-	{
-		LogDebugEntry *E = (LogDebugEntry*)vptr;
-		E->F(lset);
-		cprintf("%d %s|%d\n", CIT_OK, which, lset);
-	}
-	else {
-		cprintf("%d Log setting %s not known\n", 
-			ERROR, which);
-	}
-}
 void CtdlDestroyDebugTable(void)
 {
 
@@ -795,7 +730,7 @@ void CtdlDestroyUserHooks(void)
 }
 
 
-void CtdlRegisterMessageHook(int (*handler)(struct CtdlMessage *),
+void CtdlRegisterMessageHook(int (*handler)(struct CtdlMessage *, recptypes *),
 				int EventType)
 {
 
@@ -813,7 +748,7 @@ void CtdlRegisterMessageHook(int (*handler)(struct CtdlMessage *),
 }
 
 
-void CtdlUnregisterMessageHook(int (*handler)(struct CtdlMessage *),
+void CtdlUnregisterMessageHook(int (*handler)(struct CtdlMessage *, recptypes *),
 		int EventType)
 {
 	MessageFunctionHook *cur, *p, *last;
@@ -1444,7 +1379,7 @@ void PerformUserHooks(ctdluser *usbuf, int EventType)
 	}
 }
 
-int PerformMessageHooks(struct CtdlMessage *msg, int EventType)
+int PerformMessageHooks(struct CtdlMessage *msg, recptypes *recps, int EventType)
 {
 	MessageFunctionHook *fcn = NULL;
 	int total_retval = 0;
@@ -1462,7 +1397,7 @@ int PerformMessageHooks(struct CtdlMessage *msg, int EventType)
 	 */
 	for (fcn = MessageHookTable; fcn != NULL; fcn = fcn->next) {
 		if (fcn->eventtype == EventType) {
-			total_retval = total_retval + (*fcn->h_function_pointer) (msg);
+			total_retval = total_retval + (*fcn->h_function_pointer) (msg, recps);
 		}
 	}
 
@@ -1564,9 +1499,6 @@ CTDL_MODULE_INIT(modules)
 {
 	if (!threading) {
 		CtdlRegisterDebugFlagHook(HKEY("modules"), DebugModulesEnable, &DebugModules);
-
-		CtdlRegisterProtoHook(cmd_log_get, "LOGP", "Print Log-parameters");
-		CtdlRegisterProtoHook(cmd_log_set, "LOGS", "Set Log-parameters");
 	}
 	return "modules";
 }
