@@ -20,13 +20,13 @@ typedef void (*CtdlDbgFunction) (const int);
 extern int DebugModules;
 #define MDBGLOG(LEVEL) if ((LEVEL != LOG_DEBUG) || (DebugModules != 0))
 
-#define MOD_syslog(LEVEL, FORMAT, ...)				\
-	MDBGLOG(LEVEL) syslog(LEVEL,				\
-			      "Modules: " FORMAT, __VA_ARGS__)
+#define MOD_syslog(LEVEL, FORMAT, ...)					\
+	MDBGLOG(LEVEL) syslog(LEVEL,					\
+			      "%s Modules: " FORMAT, IOSTR, __VA_ARGS__)
 
-#define MODM_syslog(LEVEL, FORMAT)			\
-	MDBGLOG(LEVEL) syslog(LEVEL,			\
-			      "Modules: " FORMAT);
+#define MODM_syslog(LEVEL, FORMAT)				\
+	MDBGLOG(LEVEL) syslog(LEVEL,				\
+			      "%s Modules: " FORMAT, IOSTR);
 
 
 
@@ -56,6 +56,13 @@ struct CleanupFunctionHook {
 extern CleanupFunctionHook *CleanupHookTable;
 
 
+typedef struct __LogDebugEntry {
+	CtdlDbgFunction F;
+	const char *Name;
+	long Len;
+	const int *LogP;
+} LogDebugEntry;
+extern HashList *LogDebugEntryTable;
 void initialize_server_extensions(void);
 int DLoader_Exec_Cmd(char *cmdbuf);
 char *Dynamic_Module_Init(void);
@@ -75,7 +82,7 @@ void CtdlDestroyXmsgHooks(void);
 
 
 void CtdlDestroyMessageHook(void);
-int PerformMessageHooks(struct CtdlMessage *, int EventType);
+int PerformMessageHooks(struct CtdlMessage *, recptypes *recps, int EventType);
 
 
 void CtdlDestroyNetprocHooks(void);
